from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class LineItemPropertiesWithType(TcBaseObj):
    """
    The information about each BidPackageLineItem to be processed is provided by way of the
    'LineItemPropertiesWithType' data structure.
    
    :var name: Name of the BidPackageLineItem to be created.
    :var description: Description of the BidPackageLineItem Object.
    :var quote: Quote form to be added to BidPackageLineItem.
    :var bpliTypeName: Name of the type of the BidPackageLineItem to be used.
    :var liccname: Name of the LineItemConfigContext to be created.
    :var liccdesc: Description of the LineItemConfigContext to be created.
    :var partid: Id of the part to be associated with BidPackageLineItem.
    :var viewtype: PSView Type to be associated with LineItemConfigContext.
    :var quantity: Quantity to be created for BidPackageLineItem.
    :var revRule: Revision rule to be associated with LineItemConfigContext.
    :var varRule: Variant rule to be associated with LineItemConfigContext.
    :var closureRule: Closure rule to be associated with LineItemConfigContext.
    """
    name: str = ''
    description: str = ''
    quote: BusinessObject = None
    bpliTypeName: str = ''
    liccname: str = ''
    liccdesc: str = ''
    partid: str = ''
    viewtype: str = ''
    quantity: int = 0
    revRule: str = ''
    varRule: str = ''
    closureRule: str = ''



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetRelatedObjectsResponse(TcBaseObj):
    """
    The GetRelatedObjectsResponse structure contains list of RelatedObjectsInfo and service data of the operation.
    
    :var response: Response structure containing list of objects in the database for a given list of displayed objects.
    :var serviceData: ServiceData for the operation.
    """
    response: List[RelatedObjectsInfo] = ()
    serviceData: ServiceData = None


@dataclass
class RelatedObjectsInfo(TcBaseObj):
    """
    The RelatedObjectsInfo structure contains parent business object, a list of child objects and the relation name by
    which parent and children are related. The RelatedObjectsInfo datastructure is used in both input and output. In
    input RelatedObjectsInfo, childrenObj are displayed objects. In output RelatedObjectsInfo childrenObj are actual
    object in database. The parentObj and propertyName remains same in both input and output.
    
    :var parentObj: A POM_Object  that contains the list of related objects.
    :var childrenObj: A list of POM_Object related to source object.
    :var propertyName: The name of the property that relates the actual child objects to the parent. The property can
    be a relation property or reference property or empty.
    """
    parentObj: BusinessObject = None
    childrenObj: List[BusinessObject] = ()
    propertyName: str = ''



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, WorkspaceObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class AttributeFormat(TcBaseObj):
    """
    Structure representing format details of a classification attribute.
    
    :var formatType: Integer representing the attribute type, which could be one of the following:
    - - 1 = KeyLOV 
    - 0 = String 
    - 1 = Integer 
    - 2 = Real 
    - 3 = Date 
    - 4 = Time range
    - 5= Boolean
    
    
    :var formatModifier1: Integer to indicate whether the attribute is configured for one of the following - 
    - 0 - Force positive number. 
    - 1 - Accept and display either + or - 
    - 2 - Accept + or - but display - sign only.
    
    
    Note: This field will return 0 if not applicable. Only applicable if the selected formatType is 1 or 2.
    :var formatModifier2: Integer to indicate additional information about the selected formatType from one of the
    following : 
    If formatType = 0 then:
    - 0 = Print characters in upper and lower case.
    - 1 = Print all characters in upper case.
    - 2 = Print all characters in lower case.
    
    
    If formatType == 2 then:
    - Number of digits after the decimal point.
    
    
    
    Note: This field will return 0 if not applicable. Only applicable if the selected formatType is 0 or 2.
    :var formatLength: Integer representing the length of the attribute. In case of a KeyLOV (stxt), or
    Cst0KeyLOVDefintion if attribute belongs to a Classification Standard Taxonomy (CST) class, this will contain the
    KeyLOV ID.
    """
    formatType: int = 0
    formatModifier1: int = 0
    formatModifier2: int = 0
    formatLength: int = 0


@dataclass
class ClassDefinition(TcBaseObj):
    """
    The list of Classification Class or View ID / IRDIs and corresponding definition.
    
    :var childCount: Number of child classes for this class.
    :var instanceCount: Total number of classification objects instantiated in this class or any of its descendants.
    :var viewCount: Number of Views defined for this class.
    :var options: Property describing classification class' flags represented as a single integer
    To access individual property flags, a bitwise operation will be required by the client. Valid values are:
    - CLASS_falg_englishUnitBase = 1
    - CLASS_flag_isGroup = 2
    - CLASS_flag_unit_system_both = 4
    - CLASS_flag_isStorageClass = 8
    - CLASS_flag_isAssembly = 16
    
    
    :var documents: A list of attached Icons, Images and NamedRefs to this class.
    :var properties: A list of classification class properties containing property ids and their values.
    :var attributes: A list of class attributes owned by this class definition.
    """
    childCount: int = 0
    instanceCount: int = 0
    viewCount: int = 0
    options: int = 0
    documents: List[ClassDocuments] = ()
    properties: List[PropertyInfo] = ()
    attributes: List[ClassAttribute] = ()


@dataclass
class ClassDocuments(TcBaseObj):
    """
    Structure holding the attached Icon, Image or NamedRef or this child class.
    
    :var documentType: Contains type of document attached. Valid values are:
    - icon 
    - image 
    - NamedRef
    
    
    :var ticket: File ticket identifier for the attached file.
    :var originalFileName: File name for this attachment.
    """
    documentType: str = ''
    ticket: str = ''
    originalFileName: str = ''


@dataclass
class ClassParents(TcBaseObj):
    """
    Structure representing classification class' parents details.
    
    :var parents: A list of class parents' details.
    """
    parents: List[ClassDefinition] = ()


@dataclass
class ClassificationInfo(TcBaseObj):
    """
    Structure representing Classification Object details.
    
    :var classificationObj: The Classification object. If this is NULLTAG; a new classification object will be created
    otherwise existing classification object represented by 'classificationObj' will be updated.
    :var workspaceObject: The WorkspaceObject that is associated by this Classification object. If this is NULLTAG,
    then a standalone classification object was created or updated, if the classificationObj is set. Supported WSO
    types are defined by preference: 'ICS_allowable_types'.
    :var properties: List of properties containing attribute Ids and their values.
    """
    classificationObj: BusinessObject = None
    workspaceObject: WorkspaceObject = None
    properties: List[ClassificationObjectProperty] = ()


@dataclass
class ClassificationObjectInfo(TcBaseObj):
    """
    Structure representing Classification Object details.
    
    :var clsObject: The found Classification object.
    :var workspaceObject: The WorkspaceObject (WSO) that is associated by this Classification object. If this is
    NULLTAG, then the found "'clsObject'" is a standalone classification object.
    :var properties: List of properties containing, attributes ids and their values.
    """
    clsObject: BusinessObject = None
    workspaceObject: WorkspaceObject = None
    properties: List[PropertyInfo] = ()


@dataclass
class ClassificationObjectProperty(TcBaseObj):
    """
    Structure representing Classification Property which holds attribute ids and their values.
    
    :var attributeId: The ID, or IRDI if the attribute belongs to a CST class, of the attribute. For traditional
    classification attribute IDs, this is an integer that is converted into a string. For cases in which the attribute
    represents the Class ID or Class Name, this is a string representation of the identifier of those properties. These
    identifiers include:
    - CLASS_ID
    - CLASS_NAME
    - CLASS_TYPE
    - UNIT_SYSTEM
    - ATTRIBUTE_DEPENDENCY_ATTRIBUTE
    - ATTRIBUTE_DEPENDENCY_CONFIG
    - ATTRIBUTE_NAME
    - ATTRIBUTE_ID
    - USER_DATA_1
    - USER_DATA_2
    - CLASS_SHORT_NAME
    - MODIFICATION_DATE
    
    
    :var values: A list of values for this attribute in the context of a Classification object.
    For regular properties it&rsquo;s just one value. In case of VLA (variable length array) properties each value has
    its own entry.
    """
    attributeId: str = ''
    values: List[str] = ()


@dataclass
class ClassificationObjects(TcBaseObj):
    """
    Structure representing the Classification Objects of a WorkspaceObject.
    
    :var clsObjects: The list of Classification Object information.
    """
    clsObjects: List[ClassificationObjectInfo] = ()


@dataclass
class FindClassificationInfoResponse(TcBaseObj):
    """
    Structure representing the classification information returned by FindClassificationInfo operation.
    
    :var clsObjectDefs: A map (WorkspaceObject, 'ClassificationObjects') of workspace objects to a list of
    Classification objects.
    :var keyLOVDescriptors: A map (string, 'KeyLOVDef') of Classification KeyLOV IDs/IRDIs and its definitions pairs.
    :var clsClassDescriptors: A map (string, 'ClassDefinition') of Classification classes or view IDs/IRDIs and its
    corresponding definition pairs. This map also contains class attribute information.
    :var classParents: A map (string, 'ClassParents') of Classification classes or view IDs/IRDIs and its parents'
    definition pairs.
    :var classChildren: A map (string, 'ClassChildren') of Classification classes or view IDs/IRDIs and its children's
    definition pairs.
    :var clsBlockDescriptors: A map (string, 'ClassDefinition') of Classification classes IRDIs and its corresponding
    definition pairs. This map also contains class attribute information. This map is only used for Classification
    Standard Taxonomy (CST) property block information. This map differs from 'clsClassDescriptors' as it contains the
    information for property blocks that belong to the returned classes.
    :var serviceData: Any failures will be returned in the service data list of partial errors.
    """
    clsObjectDefs: ClassificationObjectsDefMap = None
    keyLOVDescriptors: KeyLOVDefMap = None
    clsClassDescriptors: ClassDefinitionMap = None
    classParents: ClassParentsMap = None
    classChildren: ClassChildrenMap = None
    clsBlockDescriptors: BlockDefinitionMap = None
    serviceData: ServiceData = None


@dataclass
class FormatProperties(TcBaseObj):
    """
    Structure representing format details.
    
    :var formatDefinition: Attribute format definition.
    :var unitName: Unit display name associated with this attribute in this unit system.
    :var defaultValue: Default value of this Class attribute. This can be an empty string indicating no default value.
    :var minimumValue: Minimum value constraint of this Class attribute. This can be an empty string indicating no
    minimum value constraint.
    Note: Only applicable to numerical formats of attributes.
    :var maximumValue: Maximum value constraint of this Class attribute. This can be an empty string indicating no
    maximum value constraint.
    Note: Only applicable to numerical formats of attributes.
    """
    formatDefinition: AttributeFormat = None
    unitName: str = ''
    defaultValue: str = ''
    minimumValue: str = ''
    maximumValue: str = ''


@dataclass
class KeyLOVDef(TcBaseObj):
    """
    Structure representing KeyLOV (stxt or Cst0KeyLOVDefinition) definition.
    
    :var keyLOVOptions: KeyLOV (stxt or Cst0KeyLOVDefinition) options to show/hide keys. Valid values are: 
    - 0 = Show key 
    - 1 = Hide key
    
    
    :var keyLOVEntries: A list of KeyLOV (stxt or Cst0KeyLOVDefinition) entries.
    :var owningSite: Owning Site (POM_imc) of this keyLOV (stxt or Cst0KeyLOVDefinition) object.
    :var shatedSites: A list of sites (POM_imc) where this KeyLOV (stxt or Cst0KeyLOVDefinition) is shared using
    Multisite operations.
    :var keyLOVProperties: keyLOV (stxt or Cst0KeyLOVDefinition) properties containing property ids and their values.
    """
    keyLOVOptions: int = 0
    keyLOVEntries: List[KeyLOVEntry] = ()
    owningSite: BusinessObject = None
    shatedSites: List[BusinessObject] = ()
    keyLOVProperties: List[PropertyInfo] = ()


@dataclass
class KeyLOVEntry(TcBaseObj):
    """
    Structure representing KeyLOV (stxt or Cst0KeyLOVDefinition) Entry.
    
    :var keyLOVkey: String representing a Key of a KeyLOV (stxt or Cst0KeyLOVDefinition) entry.
    :var keyLOVValue: String representing a Value of the KeyLOV (stxt or Cst0KeyLOVDefinition) entry.
    :var isDeprecated: Indicating whether this KeyLOV (stxt or Cst0KeyLOVDefinition) entry is deprecated.
    If true, keyLOV entry is deprecated and cannot be used for new classifications; otherwise, it can be used for new
    classifications.
    """
    keyLOVkey: str = ''
    keyLOVValue: str = ''
    isDeprecated: bool = False


@dataclass
class PropValue(TcBaseObj):
    """
    Structure containing classification property value details.
    
    :var internalValue: Internal value stored into database.
    :var displayValue: Display value for a classification property. This will be same as that of 'internalValue' for
    all classification attributes except keyLOV (stxt) attributes as they will have "entry key" as 'internalValue' and
    "entry key entry value" or "entry value" as 'displayValue'; based on the keyLOV configuration.
    """
    internalValue: str = ''
    displayValue: str = ''


@dataclass
class PropertyInfo(TcBaseObj):
    """
    Structure representing Classification Property which holds attribute ids and their values.
    
    :var propertyId: A string representing the type attribute Id of the attribute. This could also be one of the
    following strings, which are string representations of internal ids:
    - CLASS_ID
    - CLASS_TYPE
    - CLASS_NAME
    - UNIT_SYSTEM
    - ATTRIBUTE_DEPENDENCY_ATTRIBUTE
    - ATTRIBUTE_DEPENDENCY_CONFIG
    - ATTRIBUTE_NAME
    - ATTRIBUTE_ID
    - USER_DATA_1
    - USER_DATA_2
    - CLASS_SHORT_NAME
    - MODIFICATION_DATE
    
    
    :var propertyName: The name of classification attribute or name of internal identifier for classification header
    properties like class Id, unit system etc.
    :var values: A list of values for this attribute.
    """
    propertyId: str = ''
    propertyName: str = ''
    values: List[PropValue] = ()


@dataclass
class SaveClassificationObjectsResponse(TcBaseObj):
    """
    Holds the classification objects returned by the 'saveClassificationObjects' operation.
    
    :var classificationObjects: A list of created or updated Classification objects.
    :var serviceData: Any failures will be returned in the service data list of partial errors.
    """
    classificationObjects: List[ClassificationObjectInfo] = ()
    serviceData: ServiceData = None


@dataclass
class SearchCriteria(TcBaseObj):
    """
    A structure defining the search criteria used for searching classes from classification hierarchy. Also allows
    sorting the results based on a predefined criterion.
    
    :var searchAttribute: Class attribute to be searched for. Valid values are:
    - CLASS_ID
    - CLASS_NAME
    
    
    :var searchString: Query string to search the class by.
    :var sortOption: Option to sort the returned results. Valid values are:
    - -600 : Class Id
    - -607 : Class Name
    
    """
    searchAttribute: str = ''
    searchString: str = ''
    sortOption: int = 0


@dataclass
class ClassAttribute(TcBaseObj):
    """
    Structure representing class attribute details.
    
    :var attributeId: Classification attribute Id/IRDI.
    :var metricFormat: Attribute format definition in metric unit system.
    :var nonMetricFormat: Attribute format definition in non-metric unit system.
    :var arraySize: Array size or the number of values for this attribute.
    - If single valued (nonarray), then arraySize = 0 
    - If limited multi valued (array), then arraySize > 0 corresponding to the size of the array defined in the
    attribute definition.
    - If unlimited multi valued (array), then arraySize = -1
    
    
    :var options: Attribute property flags represented as a single integer. To access individual property flags, a
    bitwise operation will be required by the client. Valid values are:
    - ATTR_vla = 1
    - ATTR_external_vla = 2 
    - ATTR_mandatory = 4
    - ATTR_protected = 8
    - ATTR_unique = 16
    - ATTR_propagated = 32
    - ATTR_localValue = 64 
    - ATTR_reference = 128
    - ATTR_auto_computed = 256
    - ATTR_hidden = 512
    - ATTR_localizable = 1024
    
    
    :var attributeProperties: A list of classification attributes' property values. These properties could be attribute
    Id, attribute Name, attribute shortName etc.
    :var attributeKeyLOVDef: Configured KeyLOV definition information for this attribute.
    """
    attributeId: str = ''
    metricFormat: FormatProperties = None
    nonMetricFormat: FormatProperties = None
    arraySize: int = 0
    options: int = 0
    attributeProperties: List[PropertyInfo] = ()
    attributeKeyLOVDef: KeyLOVDef = None


@dataclass
class ClassChildren(TcBaseObj):
    """
    Structure representing classification class' children details.
    
    :var children: A list of class' children details.
    """
    children: List[ClassDefinition] = ()


"""
The list of Classification Class or View ID / IRDIs and corresponding definition.
"""
ClassDefinitionMap = Dict[str, ClassDefinition]


"""
A map of classification class Id / IRDI to a list of its parents.
"""
ClassParentsMap = Dict[str, ClassParents]


"""
Map of workspace object to a list of its classification objects or standalone classification objects with a matching ID to that of the WSO. If an object is classified then map value will have all its classification's information. If object is not classified, but system found standalone classification objects, then the map values will have standalone classification object's information. In the case of a standalone classification, the standalone classification&rsquo;s WSO tag will be a NULLTAG.
"""
ClassificationObjectsDefMap = Dict[WorkspaceObject, ClassificationObjects]


"""
A map of classification class Id / IRDI to a list of block property class definitions. Used by Classification Standard Taxonomy (CST) when a class has block properties.
"""
BlockDefinitionMap = Dict[str, ClassDefinition]


"""
The list of KeyLOV ids and corresponding KeyLOV definition. KeyLOV id and definition will be stxt BO if the class is a from traditional classification, or from Cst0KeyLOVDefinition BO if it is from Classification Standard Taxonomy (CST).
"""
KeyLOVDefMap = Dict[str, KeyLOVDef]


"""
A map of classification class Id / IRDI to a list of its children.
"""
ClassChildrenMap = Dict[str, ClassChildren]



from __future__ import annotations

from tcsoa.gen.Manufacturing._2015_10.StructureManagement import CreateCPCResponse, CreateCPCInputInfo
from tcsoa.gen.Manufacturing._2015_10.ImportExport import ImportManufaturingFeaturesInput, ImportManufaturingFeaturesResponse
from tcsoa.utils import TcService


class ImportExportService(TcService):

    @classmethod
    def importManufacturingFeatures(cls, input: ImportManufaturingFeaturesInput) -> ImportManufaturingFeaturesResponse:
        """
        This operation imports discrete and continuous manufaturing features (MFGs) from a given PLMXML file into a
        target product structure in Teamcenter. The PLMXML contains the data about the MFGs. The data contains the name
        of the MFGs. Their transform, their parts' connection, and their form attributes. There are two types of
        manufacturing features. Continuous MFG like arcweld ( Mfg0BVRArcWeld) and discrete MFGs like weld point
        (Mfg0BVRWeldPoint) and datum point (Mfg0BVRDatumPoint). For continuous MFGs, the PLMXML also contains the JT
        information.
        
        Use cases:
        none
        """
        return cls.execute_soa_method(
            method_name='importManufacturingFeatures',
            library='Manufacturing',
            service_date='2015_10',
            service_name='ImportExport',
            params={'input': input},
            response_cls=ImportManufaturingFeaturesResponse,
        )


class StructureManagementService(TcService):

    @classmethod
    def createCollabPlanningContext(cls, cpcInput: CreateCPCInputInfo) -> CreateCPCResponse:
        """
        This service operation creates a Collaboration Planning Context (CPC) with the given input  structures that are
        to be cloned and/or referred to, and establishes a relation between input MECollaborationContext (CC) object
        and newly created CPC. 
        CPC is a CC object itself, it is a term used for mix production.
        
        Use cases:
        A user creates a CPC object in Manufacturing Process Planner (MPP) application using an existing opened CC
        structure. Subsequently a relation Mfg0MEAlternatePlanningRel is created between newly created CPC and the
        orginal CC.
        
        Use Case 1: The user opens a CC structure, select some structures available in the CC and creates a CPC.
        
        Use Case 2: A user opens a CC structure, select some of the scopes in that structure and create a CPC.
        
        Use Case 3: A user opens a CC structure, select some structures/scopes, provide whether they need to be
        referred or cloned and provide cloning parameters such as Clone Suppressed Lines, Carry Over ICs etc. to create
        a CPC.
        """
        return cls.execute_soa_method(
            method_name='createCollabPlanningContext',
            library='Manufacturing',
            service_date='2015_10',
            service_name='StructureManagement',
            params={'cpcInput': cpcInput},
            response_cls=CreateCPCResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, ItemRevision, Item
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from typing import Dict


@dataclass
class IdInfo(TcBaseObj):
    """
    This structure holds the information about the objects that will be launched to the viewer.
    
    :var id: A required parameter that references the object to be launched. If needed, launched object will be
    resolved by the server to a launch able object.
    :var item: An optional object reference of the Item containing launch able object. If this is not known, the server
    will attempt to identify the parent if it can.
    :var itemRev: An optional object reference of the ItemRevision containing launchable object. If this is not known,
    the server will attempt to identify if it can.
    :var operation: An optional parameter references the type of launch action. This controls the action the viewer
    will perform when it opens the object. The actions supported are one of following: 'Open', 'Insert', 'Merge' or
    'Interop'.  'Open' will open the object in a new window.  'Insert' will insert the object into the current window
    that has focus.  'Merge' will attempt to merge a pruned product structure with one that is already open if it can. 
    'Interop' will present a dialog that lets the user select the launch action.
    :var idAdditionalInfo: An optional parameter referencing the additional information of launched objects in form of
    key/value pair (if any).
    """
    id: BusinessObject = None
    item: Item = None
    itemRev: ItemRevision = None
    operation: str = ''
    idAdditionalInfo: KeyValueMap = None


@dataclass
class ServerInfo(TcBaseObj):
    """
    This structure holds the basic information for Teamcenter Visualization to connect to the server.
    
    :var protocol: A required parameter referencing the protocol type for connection to the server. Use http for
    standard 4 tier connections, and iiop for 2 tier deployments.
    :var hostpath: A required parameter referencing the URL to connect to the server.
    :var servermode: A required parameter referencing the servermode that controls how the connection to the server is
    made: 2 for two tier. 4 for four tier.
    :var serverAdditionalInfo: An optional parameter referencing the additional  information of the server in form of
    key/value pair (if any).
    """
    protocol: str = ''
    hostpath: str = ''
    servermode: int = 0
    serverAdditionalInfo: KeyValueMap = None


@dataclass
class SessionInfo(TcBaseObj):
    """
    This structure holds the information about the session information of the client application from where the launch
    operation was initiated.
    
    :var sessionDescriminator: Client/Server session discriminator to connect to existing specified session.  This
    allows the viewer application to share an existing server process session with the client that initiated the
    launch. If this is not specified, the viewer will present a login prompt.
    :var sessionAdditionalInfo: An optional parameter referencing the additional information of the session in form of
    key/value pair (if any).
    """
    sessionDescriminator: str = ''
    sessionAdditionalInfo: KeyValueMap = None


@dataclass
class UserAgentDataInfo(TcBaseObj):
    """
    This structure holds the information about the client application that initiated the launch.
    
    :var userApplication: An optional parameter referencing the client who initiates the launch.
    :var userAppVersion: An optional parameter referencing the version of the client that initiated launch.
    :var userAdditionalInfo: An optional parameter referencing the additional information of client application in form
    of key/value pair (if any).
    """
    userApplication: str = ''
    userAppVersion: str = ''
    userAdditionalInfo: KeyValueMap = None


@dataclass
class VVITicketsResponse(TcBaseObj):
    """
    Used to return information from the 'createLaunchFile' operation. The structure holds the 'serviceData' object and
    a FMS transient read ticket corresponding to the launch file (VVI or VFZ).
    
    :var ticket: The FMS transient read ticket of the launch file (VVI or VFZ) generated for the objects that can be
    launched. The file will be placed in the transient file volume and the caller will need to download it from there
    with the ticket sent by the service.
    :var serviceData: SOA Framework object containing error information. In cases where objects cannot be launched,
    error message, codes will be mapped to respective object id in the list of partial errors.
    """
    ticket: str = ''
    serviceData: ServiceData = None


"""
Map of key-value name pair.
"""
KeyValueMap = Dict[str, str]



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class IdentifyImpactedObjInput(TcBaseObj):
    """
    The input values for the identifyImpactedObjects operation.
    
    :var tieSyncStatus: TIE_sync_status value to mark impacted objects.
    Valid values are positive integers, defined in core/tie/tie_privileged.h
    Examples:
    1 = TIE_sync_status_replication_pending
    15 = TIE_sync_status_revrule_refresh_pending
    :var impactingOperations: bit values Add=1, Modified=2, Deleted=4
    :var returnObjIds: Should the operation return the list of impacted object UIDs.
    If true the operation will return the list of objects identified, if false list will be empty.
    """
    tieSyncStatus: int = 0
    impactingOperations: int = 0
    returnObjIds: bool = False


@dataclass
class IdentifyImpactedObjectsResponse(TcBaseObj):
    """
    The list of impacted Revisionable Object UIDs from the identifyImpactedObjects operation.
    
    :var impactedObjectsUIDs: List that contains the impacted object UIDs due to adds/modify/deletes that happened
    after last time index.
    :var serviceData: The Service Data
    """
    impactedObjectsUIDs: List[str] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, VariantRule, RevisionRule
from typing import List
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class LicenseAttachOrDetachInput(TcBaseObj):
    """
    A structure containing license to be attached or removed and propagation context information.
    
    :var attachLicenseDetails: Structure containing data for attach license operation.
    :var detachLicenseDetails: Structure containing data for detach license operation.
    :var contextInfo: A structure of PropagationConfigurationContext.
    :var processAsynchronously: Flag indicating if this operation needs to be processed in asynchronously. A value True
    means to process it asynchronously. If the value is not set or is set to False then processing happens
    synchronously in the same request.
    """
    attachLicenseDetails: LicenseInputDetails = None
    detachLicenseDetails: LicenseInputDetails = None
    contextInfo: PropagationConfigurationContext = None
    processAsynchronously: bool = False


@dataclass
class LicenseInputDetails(TcBaseObj):
    """
    A Structure containing list of selected licenses and objects with the ead paragraph if there is any.
    
    :var selectedLicenses: List of license IDs of ADA licenses. These are strings of each with a maximum of 128 bytes
    size.
    :var objects: List of WorkspaceObject business objects to be either attached or detached from selected list of
    Licenses
    :var eadParagraph: List of authorizing paragraphs for the licenses being attached to WorkspaceObject business
    objects. These are strings with a maximum of 80 bytes size. The size of eadParagraph vector should match the size
    of the selectedLicenses (each entry in eadParagraph maps to corresponding entry in selectedLicenses). If a
    paragraph entry is not applicable for a specific license (paragraph entries are applicable only for licenses of
    ITAR type), then that entry can be left blank. System will ignore any paragraph entry if it is not applicable for a
    license to be attached. This is an optional parameter.
    """
    selectedLicenses: List[str] = ()
    objects: List[BusinessObject] = ()
    eadParagraph: List[str] = ()


@dataclass
class PropagationConfigurationContext(TcBaseObj):
    """
    PropagationConfigurationContext contains data which can be applied to attach to projects and detach from license
    operation.
    
    :var selectedTopLevelObject: Selected top level object in context of ACE ( Active Content Expericence ), if this
    object is passed we will fetch variant rule and revision rule.
    :var variantRule: 
    The VariantRule in context of the assign to project  or attach to license operation.
    :var revisionRule: The RevisionRule associated with the assign to project or attach to license operation.
    :var typeOption: An integer indicating the type of Item or Item Revision, valid values are 0 for Item and 1 for
    Item Revision.
    :var depth: A number indicating how deep the traversal needs to be performed for a given structure, applicable only
    for structures.
    """
    selectedTopLevelObject: BusinessObject = None
    variantRule: VariantRule = None
    revisionRule: RevisionRule = None
    typeOption: int = 0
    depth: int = 0



from __future__ import annotations

from tcsoa.gen.BusinessObjects import GroupMember, User
from typing import List
from tcsoa.gen.Server import ServiceData, Preferences
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetAvailableServicesResponse(TcBaseObj):
    """
    List of available services and operations.
    
    :var serviceNames: List of available services
    """
    serviceNames: List[str] = ()


@dataclass
class GetGroupMembershipResponse(TcBaseObj):
    """
    Info from 'getGroupMembership' operation.
    
    :var groupMembers: A list of all the valid GroupMember objects for the current session`s User.  A GroupMember holds
    identifiers for a User, Group, and Role and represents that the user belongs to a group with the particular role.
    :var serviceData: The plain list has all the GroupMembers, Groups and Roles for this User
    """
    groupMembers: List[GroupMember] = ()
    serviceData: ServiceData = None


@dataclass
class GetSessionGroupMemberResponse(TcBaseObj):
    """
    Information returned from the 'getSessionGroupMember' service operation.
    
    :var groupMember: The GroupMember object which represents the logged in user's, Group, and Role for the current
    session.
    :var serviceData: The GroupMember object is included in the plain list.
    """
    groupMember: GroupMember = None
    serviceData: ServiceData = None


@dataclass
class LoginResponse(TcBaseObj):
    """
    The  User and GroupMember objects for the user of this session. Partial errors are returned in the 'ServiceData'
    when the authentication is successful but requested role is not supported.
    
    :var user: The User of this session.
    :var groupMember: The GroupMember of this session.
    :var serviceData: The GroupMember and User are added to the plain object list.
    """
    user: User = None
    groupMember: GroupMember = None
    serviceData: ServiceData = None


@dataclass
class PrefSetting(TcBaseObj):
    """
    Info for setting preferences
    
    :var prefScope: The scope in which the preferences are to be set,
    "all", "site", "user", "group", or "role".
    :var prefName: The name of the preference.
    :var prefValues: The array of values for this perference.
    """
    prefScope: str = ''
    prefName: str = ''
    prefValues: List[str] = ()


@dataclass
class PreferencesResponse(TcBaseObj):
    """
    Info from get/setPreferences
    
    :var preferences: The requested preference name/values.
    :var serviceData: Any partial errors that may occur when filling this request.
    """
    preferences: Preferences = None
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.ProjectManagementAw._2019_06.ScheduleManagementAw import ClaimAssignmentData, LoadAssignedTasksResponse, LoadAssignedTasksInfo
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class ScheduleManagementAwService(TcService):

    @classmethod
    def claimAssignments(cls, claimAssignmentsData: List[ClaimAssignmentData]) -> ServiceData:
        """
        This operation claims assignments by deleting the resource pool assignments and assinging them to the logged in
        user. An assignment can be claimed only if the ScheduleTask object is assigned to any member in the
        ResourcePool object and the logged in user is a member of the ResourcePool object.
        """
        return cls.execute_soa_method(
            method_name='claimAssignments',
            library='ProjectManagementAw',
            service_date='2019_06',
            service_name='ScheduleManagementAw',
            params={'claimAssignmentsData': claimAssignmentsData},
            response_cls=ServiceData,
        )

    @classmethod
    def loadAssignedTasks(cls, loadTasksInfo: List[LoadAssignedTasksInfo]) -> LoadAssignedTasksResponse:
        """
        Loads the ScheduleTask objects assigned to the logged in user based on the information provided in the    
        &lsquo;LoadAssignedTasksInfo&rsquo; inputs. The assigned tasks are determined by the ResourceAssigment relation
        where the  primary object is ScheduleTask and the secondary is the current logged in User.
        """
        return cls.execute_soa_method(
            method_name='loadAssignedTasks',
            library='ProjectManagementAw',
            service_date='2019_06',
            service_name='ScheduleManagementAw',
            params={'loadTasksInfo': loadTasksInfo},
            response_cls=LoadAssignedTasksResponse,
        )



from __future__ import annotations

from enum import Enum
from typing import List
from tcsoa.gen.BusinessObjects import TaskDependency, ScheduleMember, Schedule, SchDeliverable, ResourceAssignment, ScheduleTask
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class FromSchedule(TcBaseObj):
    """
    The information regarding a schedule to copy from.
    
    :var schedule: The schedule to copy from.
    :var tasks: The list of tasks in that schedule to copy.
    :var schmgtOptions: Option structure for list of logical, integer and string options IntegerOptions {
    numberOfCopies - the number of copies } LogicalOptions { includeAssignments - copy assignments,includeDependencies
    - copy dependencies,includeDeliverables - copy deliverables,useExistingDeliverables - link copied to existing (by
    name), createMembership - copy missing members, resetDeliverableInstances - reset the instance of Schedule
    Deliverables, copyWorkflowTemplate - copy workflow template information, copyCostOfTask - copy costs,
    offsetBasedOnPositionInTemplate - shift tasks relative of original offset, recalcAsyncDispatcher - recalculate in
    asynchronously }
    """
    schedule: Schedule = None
    tasks: List[ScheduleTask] = ()
    schmgtOptions: SchMgtOptions = None


@dataclass
class MultiSchSpecialCopyResponse(TcBaseObj):
    """
    Return type of SOA for special paste
    
    :var createdTasks: Created Tasks
    :var modifiedTasks: Modified Tasks
    :var createdMembers: Created Members
    :var createdDeliverables: Created Schedule Deliverables
    :var createdDependencies: Created Dependencies
    :var createdAssignments: Created Assignments
    :var serviceData: Service Data
    """
    createdTasks: List[ScheduleTask] = ()
    modifiedTasks: List[ScheduleTask] = ()
    createdMembers: List[ScheduleMember] = ()
    createdDeliverables: List[SchDeliverable] = ()
    createdDependencies: List[TaskDependency] = ()
    createdAssignments: List[ResourceAssignment] = ()
    serviceData: ServiceData = None


@dataclass
class SchMgtIntegerOption(TcBaseObj):
    """
    Integer Options for special copy-paste
    
    :var name: The name of the option.
    :var value: The integer value of the option.
    """
    name: str = ''
    value: int = 0


@dataclass
class SchMgtLogicalOption(TcBaseObj):
    """
    Logical options for special copy-paste
    
    :var name: The name of the option.
    :var value: The value of logical option (True or False).
    """
    name: str = ''
    value: bool = False


@dataclass
class SchMgtOptions(TcBaseObj):
    """
    Special copy-paste options
    
    :var logicalOptions: List of logical options
    :var integerOptions: List of integer options
    :var stringOptions: List of string options
    """
    logicalOptions: List[SchMgtLogicalOption] = ()
    integerOptions: List[SchMgtIntegerOption] = ()
    stringOptions: List[SchMgtStringOption] = ()


@dataclass
class SchMgtStringOption(TcBaseObj):
    """
    String Options for special copy-paste
    
    :var name: The name of the option.
    :var value: The string value of the option
    """
    name: str = ''
    value: str = ''


@dataclass
class SpecialCopyContainer(TcBaseObj):
    """
    Input container
    
    :var fromSchedule: List of FromSchedules
    :var toSchedule: Target Schedule
    """
    fromSchedule: List[FromSchedule] = ()
    toSchedule: ToSchedule = None


@dataclass
class ToSchedule(TcBaseObj):
    """
    Target Schedule under which tasks will be pasted.
    
    :var targetSchedule: The target Schedule
    :var task: The target Task
    :var pasteType: Paste Type - PasteBeforeTask/PasteUnderTask/PasteAfterTask
    """
    targetSchedule: Schedule = None
    task: ScheduleTask = None
    pasteType: PasteType = None


class PasteType(Enum):
    """
    Enums which describes where to paste the copied tasks.
    """
    PasteBeforeTask = 'PasteBeforeTask'
    PasteUnderTask = 'PasteUnderTask'
    PasteAfterTask = 'PasteAfterTask'



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from enum import Enum
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ExtendedProperties(TcBaseObj):
    """
    Extended metadata properties for an attribute.
    
    :var propName: An extended prop name.
    :var propValue: Extended prop values.
    """
    propName: str = ''
    propValue: List[str] = ()


@dataclass
class AttrDescriptor(TcBaseObj):
    """
    Structure containing map of class attribute descriptors.
    
    :var attrDescMap: Map of attribute descriptors.
    """
    attrDescMap: AttributeDescriptorMap = None


@dataclass
class AttributeValues(TcBaseObj):
    """
    Contains values for ico attributes and boolean to check whether attribute is valid.
    
    :var isAttrValid: Flag to indicate whethe the attribute is valid
    :var values: List of values for this attribute
    """
    isAttrValid: bool = False
    values: List[Value] = ()


@dataclass
class Value(TcBaseObj):
    """
    structure for each ico attribute value.
    
    :var attrValue: Value of the attribute
    :var unit: Unit of the attribute.
    :var locale: Locale for this attribute.
    """
    attrValue: str = ''
    unit: str = ''
    locale: str = ''


@dataclass
class ClassAttrDescriptor(TcBaseObj):
    """
    Structure representing Class attribute description.
    
    :var id: An attribute id.
    :var name: An attribute name.
    :var metricMinValue: An attribute metric min value.
    :var metricMaxValue: An attribute metric max value.
    :var nonMetricFormat: An attribute non metric format.
    :var nonMetricUnit: An attribute non metric unit.
    :var nonMetricDefaultValue: An attribute non metric default value.
    :var nonMetricMinValue: An attribute non metric min value.
    :var nonMetricMaxValue: An attribute non metric max value.
    :var extendedProps: Extended properties.
    :var shortName: An attribute shortName.
    :var description: An attribute description.
    :var annotation: An attribute annotation.
    :var arraySize: An attribute array size.
    :var options: An attribute options.
    :var metricFormat: An attribute metric format.
    :var metricUnit: An attribute metric unit.
    :var metricDefaultValue: An attribute metric default value.
    """
    id: int = 0
    name: str = ''
    metricMinValue: str = ''
    metricMaxValue: str = ''
    nonMetricFormat: int = 0
    nonMetricUnit: str = ''
    nonMetricDefaultValue: str = ''
    nonMetricMinValue: str = ''
    nonMetricMaxValue: str = ''
    extendedProps: List[ExtendedProperties] = ()
    shortName: str = ''
    description: str = ''
    annotation: str = ''
    arraySize: int = 0
    options: int = 0
    metricFormat: int = 0
    metricUnit: str = ''
    metricDefaultValue: str = ''


@dataclass
class ClassAttrInfo(TcBaseObj):
    """
    Structure containing classification object information.
    
    :var cid: Classification class Id.
    :var attrValuesMap: Class attributes map
    :var unitBase: Class unit system of measure.
    :var icoId: Reference of the classified 'WorkspaceObject' object. This can be a NULLTAG if it is a standalone ICO.
    :var wsoTag: The tag of classifying WSO.
    """
    cid: str = ''
    attrValuesMap: AttributeValuesMap = None
    unitBase: ClassUnitBase = None
    icoId: str = ''
    wsoTag: BusinessObject = None


@dataclass
class ClassificationInfoResponse(TcBaseObj):
    """
    Contains classification objects info returned by getClassificationObjectInfo() method.
    
    :var classificationObjectInfo: Map of classification objects and list of references to their class attributes.
    :var classAttributeDesc: Map of classification objects and list of references to their attribute descriptors.
    :var svcData: Any failures with classification object ID mapped to the error message in the 'ServiceData' list of
    partial errors.
    """
    classificationObjectInfo: ClassificationObjectInfoMap = None
    classAttributeDesc: ClassAttributeDescMap = None
    svcData: ServiceData = None


class ClassUnitBase(Enum):
    """
    Enumerates the list of value for class unit system.
    """
    CLS_METRIC = 'CLS_METRIC'
    CLS_NONMETRIC = 'CLS_NONMETRIC'
    CLS_BOTH = 'CLS_BOTH'


"""
Contains descriptor of an attribute.
"""
AttributeDescriptorMap = Dict[int, ClassAttrDescriptor]


"""
Contains attriute Id and corresponding value(s).

Structure elements:

- primary key - Attribute ID.
- values - Value structure for this attribute.


"""
AttributeValuesMap = Dict[int, AttributeValues]


"""
Contains class Id and corresponding attribute descriptor.
"""
ClassAttributeDescMap = Dict[str, AttrDescriptor]


"""
Contains classification object information.

Structure elements:

- primary key - Classification object (ICO) ID.
- values - Reference of the structure containing additional ICO properties


"""
ClassificationObjectInfoMap = Dict[str, ClassAttrInfo]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetExportTemplateResponse(TcBaseObj):
    """
    The GetExportTemplateResponse structure contains the information about type of template and the display name of the
    template. The type of template can be "SpecTemplate" or "ExcelTemplate".
    
    :var serviceData: The Service data which may contain the following partial errors.
    224013 : The selected input object is not an business object of type Item or ItemRevision.
    224014 : The valid values for templateTypes are "SpecTemplate", "ExcelTemplate" or both.
    :var outTmplNames: The type of template as key and a vector of  
    templates found in the system as value. Example- The  
    "SpecTemplate" as the key and a list of all the SpecTemplates found in the database as the value. The
    "ExcelTemplate" as the key and a list of all the ExcelTemplates found in the database as the value. The
    "SpecTemplate" and "ExcelTemplate" are objects of type "SpecTemplate" and "ExcelTemplate" respectively.
    """
    serviceData: ServiceData = None
    outTmplNames: TemplateTypeVsTemplateNames = None


@dataclass
class GetTemplateInput(TcBaseObj):
    """
    The datastructure is used to collect input parameters like selected objects, type of template to be retrieved and
    additional information used for filtering.
    
    :var inputObjects: The selected objects for which the export templates are to be retrieved. The selected business
    object can be any type of Item or ItemRevision.
    :var templateTypes: The type of template to fetch, Value can be: "SpecTemplate","ExcelTemplate", or both.
    :var requestPref: A map of key and value pairs  (string, string) used for filtering and sorting. This is used as a
    future placeholder to implement template filtering and sorting. Currently its value will be blank.
    """
    inputObjects: List[BusinessObject] = ()
    templateTypes: List[str] = ()
    requestPref: RequestPreference = None


"""
The map which can have a key and value pair, used for options for filtering and sorting of export templates. The key and value are case sensitive.
"""
RequestPreference = Dict[str, str]


"""
This map will hold the type of template as key and vector of name of all templates found in the system with this name as value. Example - The SpecTemplate or "ExcelTemplate" as the key and a vector of all the template as the value.
"""
TemplateTypeVsTemplateNames = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.DocMgmtAw._2017_06.DocMgmt import PrinterDefinitionResponse
from tcsoa.utils import TcService


class DocMgmtService(TcService):

    @classmethod
    def getPrinterDefinitions(cls) -> PrinterDefinitionResponse:
        """
        This operation returns Print Configuration definition information from the PrintConfiguration object defined in
        Teamcenter.
        
        Use cases:
        The client wants to get the print configuration information required for the Batch Printing action in Active
        Workspace.
        """
        return cls.execute_soa_method(
            method_name='getPrinterDefinitions',
            library='DocMgmtAw',
            service_date='2017_06',
            service_name='DocMgmt',
            params={},
            response_cls=PrinterDefinitionResponse,
        )



from __future__ import annotations

from typing import Dict, List
from tcsoa.gen.BusinessObjects import ListOfValues
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class AttachedLOVsResponse(TcBaseObj):
    """
    AttachedLOVsResponse
    
    :var inputTypeNameToLOVOutput: Map of input type name to LOVOutput
    :var serviceData: ServiceData which has output tags as plain objects and errors in partialError
    """
    inputTypeNameToLOVOutput: InputTypeNameToLOVOutputMap = None
    serviceData: ServiceData = None


@dataclass
class LOVInfo(TcBaseObj):
    """
    LOVInfo
    
    :var typeName: The name of the Teamcenter Engineering type to which property belongs
    :var propNames: List of Property names to which the LOV is attached
    """
    typeName: str = ''
    propNames: List[str] = ()


@dataclass
class LOVOutput(TcBaseObj):
    """
    LOVOutput
    
    :var propName: Input Property name to which the LOV is attached
    :var lov: The attached LOV tag found for the input type and property name
    """
    propName: str = ''
    lov: ListOfValues = None


"""
InputTypeNameToLOVOutputMap
"""
InputTypeNameToLOVOutputMap = Dict[str, List[LOVOutput]]



from __future__ import annotations

from tcsoa.gen.Internal.BusinessModeler._2007_01.DataModelManagement import ImportDataModelResponse, ExportDataModelResponse
from tcsoa.utils import TcService


class DataModelManagementService(TcService):

    @classmethod
    def importDataModel(cls, custTemplateFileTicket: str, custDependencyFileTicket: str, updateOption: str, runMode: str) -> ImportDataModelResponse:
        """
        Imports data model to database from xml file. This operation is replaced by 'deployDataModel2'.
        """
        return cls.execute_soa_method(
            method_name='importDataModel',
            library='Internal-BusinessModeler',
            service_date='2007_01',
            service_name='DataModelManagement',
            params={'custTemplateFileTicket': custTemplateFileTicket, 'custDependencyFileTicket': custDependencyFileTicket, 'updateOption': updateOption, 'runMode': runMode},
            response_cls=ImportDataModelResponse,
        )

    @classmethod
    def exportDataModel(cls, mode: str) -> ExportDataModelResponse:
        """
        Exports data model to an xml file.
        
        Use cases:
        Teamcenter data model definitions are externally managed in XML format, and are often refered to as Business
        Modeler IDE template definitions. With this operation, the user can export the data model definitions from the
        Teamcenter database to an XML file. This is equivalent to running the business modeler extractor utility. There
        are two different modes ('schema' and 'all') that  export a different amount of information. The 'schema' mode
        will extract the application, classes, and attribute information, whereas 'all' will extract the entire data
        model information including the schema.
        """
        return cls.execute_soa_method(
            method_name='exportDataModel',
            library='Internal-BusinessModeler',
            service_date='2007_01',
            service_name='DataModelManagement',
            params={'mode': mode},
            response_cls=ExportDataModelResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Dataset, ImanFile
from tcsoa.gen.Core._2015_10.FileManagement import FileDigestInfoResponse, DatasetDigestInfoResponse
from typing import List
from tcsoa.gen.Core._2006_03.Session import LoginResponse
from tcsoa.gen.Core._2015_10.DataManagement import ReassignParticipantResponse, ReassignParticipantInfo
from tcsoa.gen.Server import ServiceData, TypeSchema
from tcsoa.gen.Core._2007_12.Session import StateNameValue
from tcsoa.utils import TcService
from tcsoa.gen.Core._2015_10.Session import TypeDescriptionOptions


class SessionService(TcService):

    @classmethod
    def getTypeDescriptions2(cls, typeNames: List[str], options: TypeDescriptionOptions) -> TypeSchema:
        """
        Gets the Meta data for the named Business Model object types based on the configurations specified by the
        client.  To improve performance, clients can specify to exclude certain Meta data such as LOV References and
        Naming Rule References for the given types.
        If options are not provided this operation returns all meta data associated with given types.
        """
        return cls.execute_soa_method(
            method_name='getTypeDescriptions2',
            library='Core',
            service_date='2015_10',
            service_name='Session',
            params={'typeNames': typeNames, 'options': options},
            response_cls=TypeSchema,
        )

    @classmethod
    def setUserSessionStateAndUpdateDefaults(cls, pairs: List[StateNameValue]) -> ServiceData:
        """
        This operation sets the desired user session state values. This operation also updates the default value of the
        Group, Role and Project when these session states are changed.
        
        To clear a field's value, client needs to pass an empty string "" as the value.
        
        Failure to set a particular state value will result in a Partial Error with the clientId set to the name of the
        state property. State values can be per client session or per server session. Client session state is kept
        separate for each client application sharing the same Teamcenter server session, while server session state is
        shared with all client application sharing the Teamcenter server session.
        
        Valid keys for the session state pairs are:
        
        - currentChangeNotice - The UID of the ChangeNotice business object for this session (client session).This is
        deprecated from release Teamcenter 11.5.
        - refreshPOM - If true the business objects in the POM are refreshed before returning property values.  This
        ensures property data is up-to-date, but is a performance hit (client session).
        - objectPropertyPolicy - The name of the current object property policy. This can also be controlled through
        the ObjectPropertyPolicyManager in the SOA client framework  (client session).
        - maxOperationBracketTime - Time (seconds) to bracket to limit a  POM refresh (client session).
        - maxOperationBracketInactiveTime - Time (seconds) to bracket to limit a  POM refresh (client session).
        - usePolicyOnly - If true, only properties defined in the current Object Property Policy will be returned.
        Objects that are added to the updated list of the ServiceData without named properties by default are returned
        with all properties currently loaded in the POM.
        - formatProperties - If true, the display value of the property will be formatted, if there is an active
        property formatter is attached to it. If false, the display value of the property will not be formatted, even
        if there is an active property formatter attached to it (client session).
        - currentProject - The UID of the Project object (server session).
        - workContex - The UID of the WorkContext object (server session).
        - volume - The UID of the Volume object (server session).
        - local_volume - The UID of the LocalVolume object (server session).
        - groupMember - The UID of the GroupMember object (server session).
        - currentDisplayRule - The UID of the DisplayRule object (server session).
        - currentOrganization - The UID of the Organization object (server session).
        - locationCodePref - The CAGE/Location Code preference. This value is set on the Item attribute
        'fnd0OriginalLocationCode' when Item objects are created (client session).
        - currentChangeItem - The UID of the Change Item Revision business object for this session. This functionality
        is supported from Teamcenter 11.5.
        
        """
        return cls.execute_soa_method(
            method_name='setUserSessionStateAndUpdateDefaults',
            library='Core',
            service_date='2015_10',
            service_name='Session',
            params={'pairs': pairs},
            response_cls=ServiceData,
        )

    @classmethod
    def sponsoredLogin(cls, sponsoringUser: str, password: str, sponsoredUser: str, sponsoredGroup: str, sponsoredRole: str, locale: str, sessionDiscriminator: str) -> LoginResponse:
        """
        Authenticates the sponsoring user`s credentials and initialize a Teamcenter session for the sponsored user. The
        operation will throw an InvalidCredentialsException as described below.
        
        When the client application is deployed to a 4Tier environment the login operation also contributes to the
        assignment of a Teamcenter server instance to the client session. The sponsoring user, sponsored users and
        sessionDiscriminator are considered in server assignment. 
        Note: The sessionDiscriminator could be blank ("") or have some value (e.g. "session1" or "session2")
        
        Example with blank ("") sessionDiscriminator: 
        - Sponsor1/User1 and Sponsor1/User2 will be assigned to different servers since their access controls are based
        on User1 and User2
        - Sponsor1/User1 and Sponsor2/User1 will be assigned to different servers.
        - Sponsor1/User1 and User1 will be assigned to different servers.
        
        Example with "session1" and "session2" sessionDiscriminator:
        - Sponsor1/User1(session1) and Sponsor1/User1(session1) will be assigned to same servers
        - Sponsor1/User1(session1) and Sponsor1/User1(session2) will be assigned to different servers
        
        Exceptions:
        >When the credentials supplied for the sponsoring user are invalid or the sponsored user's group is invalid or
        the requested locale is not allowed:
        515143: The logon was refused due to invalid username or password
        515144: The logon was refused due to invalid username or password
        515142: The logon was refused due to an invalid group for the sponsored user.
        128001: The logon was refused due to conflict with the encoding of the database instance.
        128002: The logon was refused due to missing localization.
        When the sponsoring user's credentials are valid but this user doesn't belong to "Sponsor" group.
        515145: The logon was refused since sponsoring user did not belong to "Sponsor" group.
        When the sponsoring user's credentials are valid but the userid requesting sponsorship is un-sponsorable.
        515146: The logon was refused since the userid requesting sponsorship is un-sponsorable.
        """
        return cls.execute_soa_method(
            method_name='sponsoredLogin',
            library='Core',
            service_date='2015_10',
            service_name='Session',
            params={'sponsoringUser': sponsoringUser, 'password': password, 'sponsoredUser': sponsoredUser, 'sponsoredGroup': sponsoredGroup, 'sponsoredRole': sponsoredRole, 'locale': locale, 'sessionDiscriminator': sessionDiscriminator},
            response_cls=LoginResponse,
        )

    @classmethod
    def sponsoredLoginSSO(cls, sponsoringUser: str, ssoCredentials: str, sponsoredUser: str, sponsoredGroup: str, sponsoredRole: str, locale: str, sessionDiscriminator: str) -> LoginResponse:
        """
        Authenticates the sponsoring user using Single-Sign-On (SSO) credentials and initialize a Teamcenter session
        for the sponsored user client. The username and ssoCredentials arguments are for the sponsoring user and must
        be obtained from Teamcenter's Security Services. The SsoCredentials class offers a simple API to get these
        values. The operation will throw an InvalidCredentialsException as described below.
        
        When the client application is deployed to a 4Tier environment the login operation also contributes to the
        assignment of a Teamcenter server instance to the client session. The sponsoring user, sponsored users and
        sessionDiscriminator are considered in server assignment. 
        Note: The sessionDiscriminator could be blank ("") or have some value (e.g. "session1" or "session2")
        
        Example with blank ("") sessionDiscriminator: 
        - Sponsor1/User1 and Sponsor1/User2 will be assigned to different servers since their access controls are based
        on User1 and User2
        - Sponsor1/User1 and Sponsor2/User1 will be assigned to different servers.
        - Sponsor1/User1 and User1 will be assigned to different servers.
        
        Example with "session1" and "session2" sessionDiscriminator:
        - Sponsor1/User1(session1) and Sponsor1/User1(session1) will be assigned to same servers
        - Sponsor1/User1(session1) and Sponsor1/User1(session2) will be assigned to different servers
        
        Example with "session1" and "session2" sessionDiscriminator:
        - Sponsor1/User1(session1) and Sponsor1/User1(session1) will be assigned to same servers
        - Sponsor1/User1(session1) and Sponsor1/User1(session2) will be assigned to different servers
        
        Exceptions:
        >When the credentials supplied for the sponsoring user are invalid or the sponsored user's group is invalid or
        the requested locale is not allowed:
        515143: The logon was refused due to invalid username or password
        515144: The logon was refused due to invalid username or password
        515142: The logon was refused due to an invalid group for the sponsored user.
        128001: The logon was refused due to conflict with the encoding of the database instance.
        128002: The logon was refused due to missing localization.
        When the sponsoring user's credentials are valid but this user doesn't belong to "Sponsor" group.
        515145: The logon was refused since sponsoring user did not belong to "Sponsor" group.
        When the sponsoring user's credentials are valid but the userid requesting sponsorship is un-sponsorable.
        515146: The logon was refused since the userid requesting sponsorship is un-sponsorable.
        """
        return cls.execute_soa_method(
            method_name='sponsoredLoginSSO',
            library='Core',
            service_date='2015_10',
            service_name='Session',
            params={'sponsoringUser': sponsoringUser, 'ssoCredentials': ssoCredentials, 'sponsoredUser': sponsoredUser, 'sponsoredGroup': sponsoredGroup, 'sponsoredRole': sponsoredRole, 'locale': locale, 'sessionDiscriminator': sessionDiscriminator},
            response_cls=LoginResponse,
        )


class DataManagementService(TcService):

    @classmethod
    def reassignParticipants(cls, reassignParticipantInfo: List[ReassignParticipantInfo]) -> ReassignParticipantResponse:
        """
        Reassigns the participant roles from one user to another for a given list of participant types on the input
        list of objects. The Particpant for the fromAssignee User is replaced with the Particpant for the toAssignee
        User. If the toAssignee User already exists as participant, then the fromAssignee User will not be replaced.
        
        Use cases:
        For Change Management use cases, user may need to reassign participant roles on the change objects like
        analyst, change specialist etc.  This operation can be used to reassign such participants.
        """
        return cls.execute_soa_method(
            method_name='reassignParticipants',
            library='Core',
            service_date='2015_10',
            service_name='DataManagement',
            params={'reassignParticipantInfo': reassignParticipantInfo},
            response_cls=ReassignParticipantResponse,
        )


class FileManagementService(TcService):

    @classmethod
    def getDigestInfoForDatasets(cls, datasets: List[Dataset]) -> DatasetDigestInfoResponse:
        """
        Gets the file digest information for the ImanFile objects contained in the Dataset objects. These digests can
        be used by clients to check for file integrity. Clients must use the same digest algorithm (as returned by this
        operation) to compute the digest on their end and compare with the digest returned by this operation.
        The Teamcenter File Management System (FMS) computes and stores the file digests on the volume if the content
        verification feature is turned on.
        This operation returns digests for binary files only.
        """
        return cls.execute_soa_method(
            method_name='getDigestInfoForDatasets',
            library='Core',
            service_date='2015_10',
            service_name='FileManagement',
            params={'datasets': datasets},
            response_cls=DatasetDigestInfoResponse,
        )

    @classmethod
    def getDigestInfoForFiles(cls, files: List[ImanFile]) -> FileDigestInfoResponse:
        """
        Gets the file digest information for all the ImanFile objects specified using files. These digests can be used
        by clients to check for file integrity. Clients must use the same digest algorithm (as returned by this
        operation) to compute the digest on their end and compare with the digest returned by this operation.
        The Teamcenter File Management System (FMS) computes and stores the file digests on the volume if the content
        verification feature is turned on. 
        This operation returns digests for binary files only.
        """
        return cls.execute_soa_method(
            method_name='getDigestInfoForFiles',
            library='Core',
            service_date='2015_10',
            service_name='FileManagement',
            params={'files': files},
            response_cls=FileDigestInfoResponse,
        )



import logging
from random import randint

from deprecated import deprecated
from typing import List, Dict

from tcsoa.backend import RestBackend
from tcsoa.config import TcSoaConfig
from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Server import ServiceData
from tcsoa.gen.Core._2007_12.Session import StateNameValue
from tcsoa.gen.Core._2011_06.Session import Credentials
from tcsoa.gen.Core.services import DataManagementService, SessionService


class TcConnection:
    def __init__(self, url: str, username: str, password: str, descrimator: str = None, insecure: bool = False):
        """
        This class aims to provide a simple interface to create a Teamcenter connection.
        Use it either like so:
            >>> conn = TcConnection('http://my.teamcenter:7001/tc', 'dcproxy', 'dcproxy')
            >>> try:
            >>>     # do TC stuff
            >>> finally:
            >>>     conn.close()
        Or with the Context Manager syntax like this:
            >>> with TcConnection('http://my.teamcenter:7001/tc', 'dcproxy', 'dcproxy'):
            >>>     # do TC stuff
        Or - if you are fearless - just like that
            >>> TcConnection('http://my.teamcenter:7001/tc', 'dcproxy', 'dcproxy')
            >>> # do TC stuff
            >>> TcSoaBasics.logout()


        :param url: Your Teamcenter URL, like 'http://my.teamcenter:7001/tc'
        :param username: The Teamcenter username to be used to connect to Teamcenter
        :param password: The Teamcenter password of the username provided to connect to Teamcenter
        :param descrimator: The descrimator to use - if not provided, a random descriminator will be used
        :param insecure: A flag for convenience to disable TLS checking
        """
        self.url = url
        self.username = username
        self.password = password
        self.descrimator = descrimator
        self.insecure = insecure

        self.connect()

    def __enter__(self):
        pass

    def connect(self):
        TcSoaConfig.backend = RestBackend(self.url)
        if self.insecure:
            TcSoaConfig.backend.session.verify = False
            import urllib3
            urllib3.disable_warnings()

        TcSoaBasics.setCredentials(self.username, self.password)
        TcSoaBasics.descrimator = self.descrimator if self.descrimator else f'TcSoa_{randint(0, 999999)}'
        login_result = TcSoaBasics.login()
        login_success = len(login_result['PartialErrors']) == 0
        if login_success:
            logging.info(f'Logged in on Server {TcSoaBasics.server} - with log file {TcSoaBasics.log_file}')
            return self
        else:
            errors = '\n'.join(login_result['PartialErrors'])
            logging.error('Error occured when logging in:\n' + errors)
            raise ConnectionError(errors)

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()

    def close(self):
        TcSoaBasics.logout()


def asd():
    with TcConnection('http://my.teamcenter:7001/tc', 'dcproxy', 'dcproxy') as f:
        pass


class TcSoaBasics:
    NULLTAG = 'AAAAAAAAAAAAAA'

    # Login configuration
    username = None
    password = None
    group = ''
    role = ''
    locale = 'en_US'
    descrimator = 'TcSoa.py'

    # Server Information
    log_file: str = None
    server: str = None

    @classmethod
    def setCredentials(cls, username, password, group='', role=''):
        cls.username = username
        cls.password = password
        cls.group = group
        cls.role = role

    @classmethod
    def login(cls):
        login_response = SessionService().login3(
            credentials=Credentials(
                user=cls.username,
                password=cls.password,
                group=cls.group,
                role=cls.role,
                locale=cls.locale,
                descrimator=cls.descrimator,
            )
        )
        cls.log_file = login_response.serverInfo['LogFile']
        cls.server = login_response.serverInfo['HostName']
        return login_response

    @classmethod
    def logout(cls):
        return SessionService().logout()

    @classmethod
    def load_objects(cls, uids: List[str]) -> Dict[str, BusinessObject]:
        if not uids:
            return dict()
        load_objs_result = DataManagementService().loadObjects(uids=uids)
        model_objects = load_objs_result['modelObjects']
        return {p: model_objects[p] for p in load_objs_result['plain']}

    @classmethod
    def load_object(cls, uid: str):
        obj = None
        for obj in cls.load_objects([uid]).values():
            break
        return obj

    @classmethod
    def getProperties(cls, objects: List[BusinessObject], properties: List[str]) -> Dict[str, BusinessObject]:
        response = DataManagementService().getProperties(
            objects=objects,
            attributes=properties
        )
        return {uid: response['modelObjects'][uid] for uid in response['plain']}

    @classmethod
    @deprecated(version="0.5.0", reason="Use the other getPropXXXX methods instead.")
    def getProperty(cls, bo: BusinessObject, prop_name: str):
        return cls.getProperties([bo], [prop_name])[bo['uid']]['props'][prop_name]

    @classmethod
    def getPropString(cls, bo: BusinessObject, prop_name: str):
        return cls.getPropStringList(bo, prop_name)[0]

    @classmethod
    def getPropStringList(cls, bo: BusinessObject, prop_name: str):
        return cls.getProperties([bo], [prop_name])[bo['uid']]['props'][prop_name]['uiValues']

    @classmethod
    def getMultiPropStringList(cls, bos: List[BusinessObject], prop_name: str):
        return {uid: obj.prop(prop_name) for uid, obj in cls.getProperties(bos, [prop_name]).items()}

    @classmethod
    def getMultiPropString(cls, bos: List[BusinessObject], prop_name: str):
        return {uid: val_list[0] for uid, val_list in cls.getMultiPropStringList(bos, prop_name).items()}

    @classmethod
    def getPropObject(cls, bo: BusinessObject, prop_name: str):
        return cls.getPropObjectList(bo, prop_name)[0]

    @classmethod
    def getPropObjectList(cls, bo: BusinessObject, prop_name: str):
        response: ServiceData = DataManagementService().getProperties(
            objects=[bo],
            attributes=[prop_name],
        )
        ref_obj_uids = response.modelObjects[bo.uid].prop(prop_name, db_value=True)
        return [response.modelObjects[uid] for uid in ref_obj_uids]

    @classmethod
    def getMultiPropObjectList(cls, bos: List[BusinessObject], prop_name: str):
        response = DataManagementService().getProperties(
            objects=bos,
            attributes=[prop_name],
        )
        obj2refs_map = {bo.uid: response.modelObjects[bo.uid].prop(prop_name, db_value=True) for bo in bos}
        return {uid: [response.modelObjects[ref] for ref in refs] for uid, refs in obj2refs_map.items()}

    @classmethod
    def getMultiPropObject(cls, bos: List[BusinessObject], prop_name: str):
        return {uid: val_list[0] for uid, val_list in cls.getMultiPropObjectList(bos, prop_name).items()}

    @classmethod
    def bool2property_str(cls, val: bool) -> str:
        return '1' if val else '0'

    @classmethod
    def setBypass(cls, enabled=True):
        return SessionService().setUserSessionState(
            pairs=[
                StateNameValue(
                    name='fnd0bypassflag',
                    value=cls.bool2property_str(enabled)
                )
            ]
        )

    @classmethod
    def set_object_load_policy(cls, bo_name: str, properties: List[str]):
        rest_backend: RestBackend = TcSoaConfig.backend
        rest_backend.set_object_load_policy(bo_name, properties)



from __future__ import annotations

from tcsoa.gen.ProjectManagement._2011_12.ScheduleManagement import DependencyCreateContainer, CreatedDependenciesContainer
from typing import List
from tcsoa.gen.BusinessObjects import Schedule
from tcsoa.utils import TcService


class ScheduleManagementService(TcService):

    @classmethod
    def createDependencies(cls, schedule: Schedule, newDependencies: List[DependencyCreateContainer]) -> CreatedDependenciesContainer:
        """
        Creates Dependencies between tasks in the same schedule, between a task and a proxy task in the same schedule,
        or between a tasks in different schedules (but in the same master schedule).  It returns the created
        dependencies, created proxy tasks (if any), and the ob
        
        Exceptions:
        >None
        """
        return cls.execute_soa_method(
            method_name='createDependencies',
            library='ProjectManagement',
            service_date='2011_12',
            service_name='ScheduleManagement',
            params={'schedule': schedule, 'newDependencies': newDependencies},
            response_cls=CreatedDependenciesContainer,
        )



from __future__ import annotations

from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ArchiveRestoreResponse(TcBaseObj):
    """
    Return structure for ArchiveRestore operation.
    
    :var reportFileName: Name of the report file generated by archive/restore operation under staging directory.
    :var reportFileTicket: The  report file FMS read ticket generated by the operation. The report file ticket will be
    empty when executed in background mode.
    :var progRecUid: Unique Identifier of the progress data record generated by the operation.
    :var serviceData: Holds any partial errors occurred during the operation.
    """
    reportFileName: str = ''
    reportFileTicket: str = ''
    progRecUid: str = ''
    serviceData: ServiceData = None


@dataclass
class GetReportTicketResponse(TcBaseObj):
    """
    Return structure for getReportFileTicket operation.
    
    :var ticket: The requested report file  FMS transient ticket.
    :var serviceData: Holds any partial errors occurred during the operation.
    """
    ticket: str = ''
    serviceData: ServiceData = None


@dataclass
class NamesAndValues(TcBaseObj):
    """
    The NamesAndValues structure represents a generic name-value pair.
    
    :var nameStr: The name of the session option.
    :var valueStr: The value of the session option.
    """
    nameStr: str = ''
    valueStr: str = ''



from __future__ import annotations

from tcsoa.gen.Administration._2016_10.UserManagement import GetCurrentCountryPageInfoResponse
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class UserManagementService(TcService):

    @classmethod
    def saveAndValidateCurrentCountry(cls, selectedcountry: str) -> ServiceData:
        """
        This operation saves the selected country value for the session user and then validates that this user
        hasn&rsquo;t violated the global license contract. If the validation of the global contract fails, the
        user&rsquo;s current country  will still be set to the selected country. Note, the selectedCountry attribute is
        not validated against the ISO 3166-1 alpha-2 standard.
        """
        return cls.execute_soa_method(
            method_name='saveAndValidateCurrentCountry',
            library='Administration',
            service_date='2016_10',
            service_name='UserManagement',
            params={'selectedcountry': selectedcountry},
            response_cls=ServiceData,
        )

    @classmethod
    def getCurrentCountryPageInfo(cls) -> GetCurrentCountryPageInfoResponse:
        """
        This operation retrieves current configuration of country selection. The configuration is a list of selectable
        countries, the initial country to be displayed, and a customer configurable confidentiality statement.
        
        Use cases:
        This will be used for both RAC and AW to populate their current country selection pages.
        """
        return cls.execute_soa_method(
            method_name='getCurrentCountryPageInfo',
            library='Administration',
            service_date='2016_10',
            service_name='UserManagement',
            params={},
            response_cls=GetCurrentCountryPageInfoResponse,
        )



from __future__ import annotations

from tcsoa.gen.Ai._2012_09.Ai import FindRequestsResponse
from tcsoa.gen.Ai._2013_12.Ai import FindRequestsWithDependencyFilter
from tcsoa.utils import TcService


class AiService(TcService):

    @classmethod
    def findRequestsWithDependencies(cls, filter: FindRequestsWithDependencyFilter) -> FindRequestsResponse:
        """
        This operation is based on dependency between the related  RequestObjects and on the input criteria provided to
        the operation input. The operation will return a list of RequestObjects to process. The returned RequestObjects
        are those whose property "type" is of value "Sync" and property "state" is of value "Pending". Further, this
        operation returns only those RequestObjects which are dependent on RequestObjects whose state is "Completed". 
        Dependency of RequestObject is determined by "fnd0pred_list" property. This property points to its preceding
        Request Object.
        """
        return cls.execute_soa_method(
            method_name='findRequestsWithDependencies',
            library='Ai',
            service_date='2013_12',
            service_name='Ai',
            params={'filter': filter},
            response_cls=FindRequestsResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, ListOfValues
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetValidRelationTypesResponse(TcBaseObj):
    """
    Response of the getValidRelationTypes SOA.
    
    :var relationTypesResults: Array of relation types results
    :var serviceData: Exceptions from internal processing returned as PartialErrors
    """
    relationTypesResults: List[RelationTypesOutput] = ()
    serviceData: ServiceData = None


@dataclass
class AttachedPropDescsResponse(TcBaseObj):
    """
    Attached Property Descriptors Response
    
    :var inputTypeNameToPropDescOutput: Map of input type name to PropertyDescriptor
    :var serviceData: ServiceData which has output tags as plain objects and errors in partialError
    """
    inputTypeNameToPropDescOutput: InputTypeNameToPropDescOutputMap = None
    serviceData: ServiceData = None


@dataclass
class PropDesc(TcBaseObj):
    """
    The PropDesc struct describes information about the Teamcenter property
    
    :var propName: Name of the property
    :var displayName: Display name of the property
    :var isEnabled: Specifies whether the property is enabled
    :var isModifiable: Specifies whether the property is modifiable
    :var attachedSpecifier: attachedSpecifier
    :var maxLength: maxLength
    :var interdependentProps: interdependentProps
    :var namingPatterns: Naming patterns for property, can be null
    :var defaultValue: Default value for the property
    :var propValueType: Value type for the property,
    PROP_untyped (0) No Value type
    PROP_char (1) Value is a single character
    PROP_date (2) Value is a date
    PROP_double (3) Value is a double
    PROP_float (4) Value is a float
    PROP_int (5) Value is an integer
    PROP_logical (6) Value is a logical
    PROP_short (7) Value is a short
    PROP_string (8) Value is a character string
    PROP_typed_reference (9) Value is a typed reference
    PROP_untyped_reference (10) Value is an untyped reference
    PROP_external_reference (11) Value is an external reference
    PROP_note (12) Value is a note
    PROP_typed_relation (13) Value is a typed relation
    PROP_untyped_relation (14) Value is an untyped relation
    :var propType: Type for the property
    PROP_unknown (0) Property type is Unknown
    PROP_attribute (1)  Based on a POM Attribute (int, string, ...)
    PROP_reference (2)  Based on a POM Reference
    PROP_relation (3) Based on an ImanRelation
    PROP_compound (4) Based on a property from another Type
    PROP_runtime (5) Based on a computed value
    :var isDisplayable: isDisplayable
    :var isArray: Specifies whether the property is an array or single value
    :var maxNumElems: Specifies the max number of elements
    :var lov: ListOfValues object attached to the property (if any)
    :var isRequired: Specifies whether the property is required
    """
    propName: str = ''
    displayName: str = ''
    isEnabled: bool = False
    isModifiable: bool = False
    attachedSpecifier: int = 0
    maxLength: int = 0
    interdependentProps: List[str] = ()
    namingPatterns: List[str] = ()
    defaultValue: str = ''
    propValueType: int = 0
    propType: int = 0
    isDisplayable: bool = False
    isArray: bool = False
    maxNumElems: int = 0
    lov: ListOfValues = None
    isRequired: bool = False


@dataclass
class PropDescInfo(TcBaseObj):
    """
    Property Description Information
    
    :var typeName: The name of the Teamcenter Engineering type to which property belongs
    :var propNames: List of Property names for which PropDesc needs to be fetched
    """
    typeName: str = ''
    propNames: List[str] = ()


@dataclass
class PropDescOutput(TcBaseObj):
    """
    Property Description Output
    
    :var propName: Input Property name for which PropDesc needs to be fetched
    :var propertyDesc: The PropDescriptor struct describes information about the Teamcenter property
    """
    propName: str = ''
    propertyDesc: PropDesc = None


@dataclass
class RelationTypeInfo(TcBaseObj):
    """
    RelationType information
    
    :var name: Name of the relation type.
    :var displayName: Display name of the relation type (localized).
    """
    name: str = ''
    displayName: str = ''


@dataclass
class RelationTypesInput(TcBaseObj):
    """
    Input structure for the getValidRelationTypes SOA.
    
    :var sourceType: Type of the source for assignment (part, tool, etc.)
    :var targetType: Type of the target for assignment (process, operation, etc.)
    """
    sourceType: BusinessObject = None
    targetType: BusinessObject = None


@dataclass
class RelationTypesOutput(TcBaseObj):
    """
    Output structure for the getValidRelationTypes SOA.
    
    :var sourceType: Type of the source for assignment (part, tool, etc.)
    :var targetType: Type of the target for assignment (process, operation, etc.)
    :var validRelationTypes: Array of PSOccurrenceType names
    """
    sourceType: BusinessObject = None
    targetType: BusinessObject = None
    validRelationTypes: List[RelationTypeInfo] = ()


"""
InputTypeNameToPropDescOutputMap
"""
InputTypeNameToPropDescOutputMap = Dict[str, List[PropDescOutput]]



from __future__ import annotations

from tcsoa.gen.Administration._2008_03.IRM import LicenseStatusResponse, DeactivateUserInput, ActivateUserInput
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class IRMService(TcService):

    @classmethod
    def deactivateUsers(cls, deactivateUser: List[DeactivateUserInput]) -> ServiceData:
        """
        This operation deactivates given users and transfers ownership of the objects owned by the users to be
        deactivated to new users if the new users are specified. The users deactivated successfully are added in the
        updated object list of the service data. If new users and groups are specified to take the ownership of the
        objects owned by the deactivated users, then the new users and groups are added in the updated object list as
        well after ownership is successfully transferred. This operation requires system administration privilege.
        """
        return cls.execute_soa_method(
            method_name='deactivateUsers',
            library='Administration',
            service_date='2008_03',
            service_name='IRM',
            params={'deactivateUser': deactivateUser},
            response_cls=ServiceData,
        )

    @classmethod
    def activateUsers(cls, activateUser: List[ActivateUserInput]) -> LicenseStatusResponse:
        """
        This operation can be used to activate user(s) based on the number of allowed active author or consumer
        licenses. If not enough licenses are available, this operation will return corresponding error code for the
        given license level. This operation activates only the user and not the Group Members corresponding to the user.
        """
        return cls.execute_soa_method(
            method_name='activateUsers',
            library='Administration',
            service_date='2008_03',
            service_name='IRM',
            params={'activateUser': activateUser},
            response_cls=LicenseStatusResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, Awp0FullTextSavedSearch
from enum import Enum
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class Awp0FullTextSavedSearchInputObject(TcBaseObj):
    """
    Input structure for createFullTextSavedSearch service operation.
    
    :var pinSearch: Should the saved search object be pinned? Supported values are:
    -1 = Do no pin or unpin the saved search.
    0 = Unpin the saved search
    1 = Pin the saved search
    :var receiveNotification: Should client receive notification if saved-search result changes? Supported values are:
    -1 = Do no set or remove notification for the saved search.
    0 = Remove notification.
    1 = Set notification.
    :var shareSavedSearch: Should the saved search be available to all users? The awp0is_global_shared property on
    Awp0FullTextSavedSearch object indicates if the search is shared or not.  
    -1 =  Do not change the value of set awp0is_global_shared property on Awp0FullTextSavedSearch object. 
    0 = set awp0is_global_shared property on Awp0FullTextSavedSearch object to false
    1 = set awp0is_global_shared property on Awp0FullTextSavedSearch object to true
    :var savedSearchName: The saved search name. Alphanumeric and special characters are allowed.
    :var searchString: The search string. Alphanumeric and special characters are allowed.
    :var searchFilterMap: A map of category internal names and list of filters in each category. (string/list of 
    SearchFilter).
    :var override: If false and if a Awp0FullTextSavedSearch object already exists for the user having same name, then
    this operation should return a partial error. If true and if a Awp0FullTextSavedsearch object already exists for
    the user having same name then this operation deletes the existing.
    """
    pinSearch: int = 0
    receiveNotification: int = 0
    shareSavedSearch: int = 0
    savedSearchName: str = ''
    searchString: str = ''
    searchFilterMap: SearchFilterMap2 = None
    override: bool = False


@dataclass
class Awp0FullTextSavedSearchInputObject2(TcBaseObj):
    """
    Input structure for modifySavedSearchProperties service operation.
    
    :var pinSearch: Should the saved-search object be pinned? Supported values are:
    -1 =  Do not pin or unpin the saved-search
    0 = Unpin the saved-search
    1 = Pin the saved-search
    :var receiveNotification: Should user receive notification if saved-search results change? Supported values are:
    -1 = Do not remove or set notification
    0 = Remove notification
    1 = Set Notification
    :var shareSavedSearch: Should the saved search be available to all users. The awp0is_global_shared property on
    Awp0FullTextSavedSearch object indicates if the search is shared or not.  
    -1 =  Do not change the value of set awp0is_global_shared property on Awp0FullTextSavedSearch object. 
    0 = set awp0is_global_shared property on Awp0FullTextSavedSearch object to false
    1 = set awp0is_global_shared property on Awp0FullTextSavedSearch object to true
    :var searchFilterMap: A map of category internal names and list of filters in each category. (string/list of
    SearchFilter).
    :var propInfo: A list of PropInfo which contains information about the objects and the properties to set.
    """
    pinSearch: int = 0
    receiveNotification: int = 0
    shareSavedSearch: int = 0
    searchFilterMap: SearchFilterMap2 = None
    propInfo: List[PropInfo] = ()


@dataclass
class Awp0FullTextSavedSearchResponse(TcBaseObj):
    """
    List of newly created saved searchesand the service data containing partial errors if any.
    
    :var savedSearchList: List of Awp0FullTextSavedSearch objects created or modified by this service operation.
    :var serviceData: Service data.
    """
    savedSearchList: List[Awp0FullTextSavedSearch] = ()
    serviceData: ServiceData = None


@dataclass
class NameValueStruct(TcBaseObj):
    """
    This structure holds property names and values for each property.
    
    :var name: Name of the property.
    :var values: Property values.
    """
    name: str = ''
    values: List[str] = ()


@dataclass
class PropInfo(TcBaseObj):
    """
    This structure holds information about object & its timestamp and property information.
    
    :var object: Awp0FullTextSavedSearch object.
    :var properties: A list of property information.
    """
    object: BusinessObject = None
    properties: List[NameValueStruct] = ()


@dataclass
class SearchFilter2(TcBaseObj):
    """
    A structure representing a string, date or numeric type of search filter.
    
    :var searchFilterType: The type of search filter. Valid values are "StringFilter", "DateFilter", "NumericFilter".
    :var stringValue: The value for a string filter. This field is applicable only if the "searchFilterType" field is
    set to "StringFilter".
    :var startDateValue: The starting value for a date filter. This field is applicable only if the "searchFilterType"
    field is set to "DateFilter".
    :var endDateValue: The ending value for a date filter. This field is applicable only if the "searchFilterType"
    field is set to "DateFilter".
    :var startNumericValue: The starting value for a numeric filter. This field is applicable only if the
    "searchFilterType" field is set to "NumericFilter".
    :var endNumericValue: The ending value for a numeric filter. This field is applicable only if the
    "searchFilterType" field is set to "NumericFilter".
    :var count: The number of values in the filter. This field is populated on the service response and is ignored on
    the service input.
    :var selected: A flag that indicates if the filter was previously selected and used to filter the search results.
    This field is populated on the service response and is ignored on the service input.
    :var startEndRange: The 'gap' used to generate the start and end values.
    """
    searchFilterType: SearchFilterType2 = None
    stringValue: str = ''
    startDateValue: datetime = None
    endDateValue: datetime = None
    startNumericValue: float = 0.0
    endNumericValue: float = 0.0
    count: int = 0
    selected: bool = False
    startEndRange: str = ''


class SearchFilterType2(Enum):
    """
    An enumeration for different types of search filters like string, date, numeric.
    """
    SearchStringFilter = 'SearchStringFilter'
    SearchDateFilter = 'SearchDateFilter'
    SearchNumericFilter = 'SearchNumericFilter'


"""
A map of category internal names and list of filters in each category. (string/list of  SearchFilter).
"""
SearchFilterMap2 = Dict[str, List[SearchFilter2]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class GetLicenseDetailsResponse(TcBaseObj):
    """
    'GetLicenseDetailsResponse' represents the details of ADA_License business objects.
    
    :var licDetails: Structures containing the properties of ADA_License business objects.
    :var serviceData: Contains partial errors encountered while fetching properties for ADA_License business objects.
    The ADA_License business objects for which properties are fetched successfully are added to the Plain list.
    """
    licDetails: List[LicenseDetails] = ()
    serviceData: ServiceData = None


@dataclass
class LicenseDetails(TcBaseObj):
    """
    'LicenseDetails' structure represents all the details of an ADA License object.
    
    :var licenseType: The type of ADA License object. The type can be ITAR_License, Exclude_License, or IP_License.
    :var licenseId: The unique ID of the license. This is string with a maximum of 128 bytes.
    :var expiryDate: The expiry date for the license, after which the license cannot be attached to WorkspaceObject
    business objects and ceases to grant the access to users/groups listed on the license. A NULL date specifies the
    license will never expire.
    :var licenseReason: A reason for the license to exist. This parameter can be a maximum of 128 bytes. This is an
    optional parameter and may have value as a blank string.
    :var users: The list of users associated with a license identified by 'licenseId'. When the license is attached to
    a classified WorkspaceObject, the users listed on the license will get access to the WorkspaceObject, based on the
    access rules.
    :var groups: List of names of groups associated with the license identified by 'licenseId'. For subgroups, the full
    names should be specified. When the license is attached to a classified WorkspaceObject, the users from the
    groups/subgroups listed on the license will get access to the workspace object, based on the access rules. This
    parameter represents an array of group name strings of upto 256 bytes in size.
    """
    licenseType: str = ''
    licenseId: str = ''
    expiryDate: datetime = None
    licenseReason: str = ''
    users: List[str] = ()
    groups: List[str] = ()


@dataclass
class LicenseIdAndType(TcBaseObj):
    """
    'LicenseIdAndType' structure Represents license Type and license ID of an ADA License object.
    
    :var licenseType: The type of a license object.  This is string with a maximum of 32 bytes.
    :var licenseId: The unique ID of the license object. This is string with a maximum of 128 bytes.
    """
    licenseType: str = ''
    licenseId: str = ''


@dataclass
class LicenseIdsAndTypesResponse(TcBaseObj):
    """
    'LicenseIdsAndTypesResponse' Represents the license IDs and types of ADA_License business objects.
    
    :var licIdType: Structures containing the license Type and license ID of fetched ADA_License business objects
    :var serviceData: Contains partial errors encountered while fetching ADA_License business objects.
    """
    licIdType: List[LicenseIdAndType] = ()
    serviceData: ServiceData = None


@dataclass
class LicenseInput(TcBaseObj):
    """
    'LicenseInput' represents a list of license IDs of ADA licenses and the associated WorkspaceObject business objects.
    
    :var selectedLicenses: List of license IDs of ADA licenses. These are strings of each with a maximum of 128 bytes
    size.
    :var objects: List of WorkspaceObject business objects associated with the 'selectedLicenses'.
    """
    selectedLicenses: List[str] = ()
    objects: List[BusinessObject] = ()


@dataclass
class LicenseTypeAndStatusFilter(TcBaseObj):
    """
    'LicenseTypeAndStatusFilter' structure Represents type and status of ADA License objects.
    
    :var licType: The type of ADA License objects. The type can be specified as ITAR_License, ExcludeE_License,
    IP_License, or, ALL. The value of ALL indicates that licenses of all types need to be fetched.
    :var licStatus: The status of ADA License objects. The value should be set to 'ALL' for returning all licenses. Any
    other value for 'licStatus' returns only unlocked and unexpired licenses.
    """
    licType: str = ''
    licStatus: str = ''



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, TraceLink
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ManagedRelationResponse(TcBaseObj):
    """
    Managed Relation Response
    
    :var managedRelationObjects: List of Managed Relation Objects
    :var serviceData: The successful Object ids, partial errors and failures
    """
    managedRelationObjects: List[TraceLink] = ()
    serviceData: ServiceData = None


@dataclass
class ModifyManagedRelationInput(TcBaseObj):
    """
    Modify Managed Relation Input
    
    :var relationTag: Tag of the Relation
    :var setSourcesInput: Modify Sources Input
    :var setTargetsInput: Modify Targets Input
    """
    relationTag: BusinessObject = None
    setSourcesInput: ModifySources = None
    setTargetsInput: ModifyTargets = None


@dataclass
class ModifySources(TcBaseObj):
    """
    Modify Sources
    
    :var addSources: List of Sources to Add
    :var removeSources: List of Sources to Remove
    """
    addSources: List[BusinessObject] = ()
    removeSources: List[BusinessObject] = ()


@dataclass
class ModifyTargets(TcBaseObj):
    """
    Modify Targets
    
    :var addTargets: List of Targets to Add
    :var removeTargets: List of Targets to Remove
    """
    addTargets: List[BusinessObject] = ()
    removeTargets: List[BusinessObject] = ()


@dataclass
class TraceabilityInfoInput(TcBaseObj):
    """
    Traceability Information Input
    
    :var inputTag: Tag of the input
    :var reportType: Report Type
    :var reportDepth: Report Depth
    """
    inputTag: BusinessObject = None
    reportType: str = ''
    reportDepth: int = 0


@dataclass
class TraceabilityReportOutput(TcBaseObj):
    """
    Traceability Report Output
    
    :var definingTree: List of Defining Reports
    :var indirectDefiningTree: List of Defining Reports (Indirect)
    :var complyingTree: List of Complying Reports
    :var indirectComplyingTree: List of Complying Reports (Indirect)
    :var serviceData: The successful Object ids, partial errors and failures
    """
    definingTree: List[DefiningReport] = ()
    indirectDefiningTree: List[DefiningReport] = ()
    complyingTree: List[ComplyingReport] = ()
    indirectComplyingTree: List[ComplyingReport] = ()
    serviceData: ServiceData = None


@dataclass
class ComplyingReport(TcBaseObj):
    """
    Complying Report
    
    :var parent: Tag of the Parent
    :var children: List of Children
    """
    parent: BusinessObject = None
    children: List[BusinessObject] = ()


@dataclass
class CreateManagedRelationInput(TcBaseObj):
    """
    This structure has all the information needed to create TraceLink managed relation.
    
    :var name: name
    :var type: Type will decide what relation to be created
    :var primaryTagList: primaryTagList
    :var secondaryTagList: secondaryTagList
    """
    name: str = ''
    type: str = ''
    primaryTagList: List[BusinessObject] = ()
    secondaryTagList: List[BusinessObject] = ()


@dataclass
class DefiningReport(TcBaseObj):
    """
    Defining Report
    
    :var parent: Tag of the Parent
    :var children: List of Children
    """
    parent: BusinessObject = None
    children: List[BusinessObject] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, PSBOMView, Item, WorkspaceObject
from enum import Enum
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class ObjectInfo(TcBaseObj):
    """
    This structure holds object information of parent object of trace link in Report structure.
    
    :var contextName: This parameter will hold the name of context for source or target object of Trace Link.
    :var displayObj: If the Trace Link is created on the occurrence then this variable will represent the object to
    show in the Trace Report.
    :var object: This object represent any Business Object in Teamcenter on which Trace Link is created
    :var bomView: This field represents the BOM View of context line for the Trace Link.
    """
    contextName: str = ''
    displayObj: WorkspaceObject = None
    object: BusinessObject = None
    bomView: PSBOMView = None


@dataclass
class Report(TcBaseObj):
    """
    This structure holds information each trace link report with information of parent object, and its children
    associated with trace link relation.
    
    :var parent: Parent object of trace link.
    :var children: list of all child objects for the parent with trace link relation
    """
    parent: ObjectInfo = None
    children: List[ObjectInfo] = ()


@dataclass
class SaveAsIn(TcBaseObj):
    """
    'SaveAsI'n structure contains the object to be saved and the corresponding 'DeepCopyData' of the attached objects
    of the object to be saved.
    
    :var targetObject: Target object being saved as
    :var saveAsInput: SaveAsInput (user input from SaveAs dialog) for the targetobject being saved as
    :var deepCopyDatas: DeepCopyData of the objects attached to the targetobject
    """
    targetObject: BusinessObject = None
    saveAsInput: SaveAsInput = None
    deepCopyDatas: List[DeepCopyData] = ()


@dataclass
class SaveAsInput(TcBaseObj):
    """
    This structure is used to capture the inputs required for savingof a business object.
    
    :var boName: Business object type name
    :var stringProps: Map of string property names to values ('string, string')
    :var boolArrayProps: Map of boolean array property names to values ('string, vector<bool>')
    :var dateProps: Map of DateTime property names to values ('string, DateTime')
    :var dateArrayProps: Map of DateTime array property names to values ('string, vector<DateTime>')
    :var tagProps: Map of BusinessObject property names to values ('string, BusinessObject')
    :var tagArrayProps: Map of BusinessObject array property names to values ('string, vector<BusinessObject>')
    :var stringArrayProps: Map of string array property names to values ('string, vector<string>')
    :var doubleProps: Map of double property names to values ('string, double')
    :var doubleArrayProps: Map of double array property names to values ('string, vector<double>')
    :var floatProps: Map of float property names to values '(string, float')
    :var floatArrayProps: Map of float array property names to values ('string, vector<float>')
    :var intProps: Map of int property names to values ('string, int')
    :var intArrayProps: Map of int array property names to values ('string, vector<int>')
    :var boolProps: Map of boolean property names to values ('string, bool')
    """
    boName: str = ''
    stringProps: StringMap1 = None
    boolArrayProps: BoolVectorMap1 = None
    dateProps: DateMap1 = None
    dateArrayProps: DateVectorMap1 = None
    tagProps: TagMap1 = None
    tagArrayProps: TagVectorMap1 = None
    stringArrayProps: StringVectorMap1 = None
    doubleProps: DoubleMap1 = None
    doubleArrayProps: DoubleVectorMap1 = None
    floatProps: FloatMap1 = None
    floatArrayProps: FloatVectorMap1 = None
    intProps: IntMap1 = None
    intArrayProps: IntVectorMap1 = None
    boolProps: BoolMap1 = None


@dataclass
class SaveAsObjectsResponse(TcBaseObj):
    """
    This structure contains a vector corresponding to the input target objects that holds mapping between the original
    objects and the saved objects.
    
    :var output: SaveAsout vector
    :var saveAsTrees: List of the input target objects that holds mapping between the original objects and the copied 
    objects.
    :var serviceData: Service data containing created objects, errors, etc
    """
    output: List[SaveAsOut] = ()
    saveAsTrees: List[SaveAsTree] = ()
    serviceData: ServiceData = None


@dataclass
class SaveAsOut(TcBaseObj):
    """
    This structure contains information for 'SaveAs' operation including unique target object.
    
    :var targetObject: BusinessObject name
    :var objects: Input data for 'SaveAs' Operation
    """
    targetObject: BusinessObject = None
    objects: List[BusinessObject] = ()


@dataclass
class SaveAsTree(TcBaseObj):
    """
    This structure contains tree structure for 'SaveAs'.
    
    :var originalObject: Original object being saved as
    :var objectCopy: Object copy after 'SaveAs'. This could be NULL if no copy was made or same as original object if
    the copy is a reference to the original
    :var childSaveAsNodes: Nested information for next level of the tree
    """
    originalObject: BusinessObject = None
    objectCopy: BusinessObject = None
    childSaveAsNodes: List[SaveAsTree] = ()


@dataclass
class TraceReport(TcBaseObj):
    """
    This structure holds information about generated trace reports with defining and complying trace trees with
    selected objects, and its persistent objects.
    
    :var definingTree: Represents the Defining Tree in the Trace Report giving all defining trace link details.
    :var indirectDefiningTree: Represents an Indirect Defining Tree in the Trace  Report.
    :var complyingTree: Represents the Complying Tree in the Trace Report giving all complying trace link details.
    :var indirectComplyingTree: Represents an Indirect Complying Tree in the Trace Report
    :var selectedObject: Represents selected object for which Trace Report generated.
    :var persistentObjects: Represents persistent object for the selected object for which Trace Report generated.
    """
    definingTree: List[Report] = ()
    indirectDefiningTree: List[Report] = ()
    complyingTree: List[Report] = ()
    indirectComplyingTree: List[Report] = ()
    selectedObject: BusinessObject = None
    persistentObjects: List[BusinessObject] = ()


@dataclass
class TraceabilityInfoInput(TcBaseObj):
    """
    Information required to generate a trace report, it includes objects for which this trace report should be
    generated, type of trace report, if indirect trace link included, and type of base relation of trace link.
    
    :var selectedObjs: List of objects on which to generate the Trace Report.
    :var reportType: Type of Report that is defining, complying, or both. Below are 
                    allowed values
                    1 for Complying Report  
                    2 for Defining Report
                    3 for Complete Report
    :var reportDepth: Level to which Trace Report should be generated. The allowed value is any number greater than
    zero.
    :var isIndirectTraceReportNeeded: If this variable is true then only Indirect Trace Report will be 
    included in the final Trace Report.
    :var relationTypeName: The Trace Report will be generated for this type of relations, the type should be
    FND_TraceLink or its subtype.
    """
    selectedObjs: List[BusinessObject] = ()
    reportType: int = 0
    reportDepth: int = 0
    isIndirectTraceReportNeeded: bool = False
    relationTypeName: str = ''


@dataclass
class TraceabilityReportOutput(TcBaseObj):
    """
    TraceabilityReportOutput structure holds information about generated trace reports with defining and complying
    trace trees with selected objects and its persistent objects, including service data.
    
    :var traceReports: This member holds all the Trace Reports user has asked for. This is vector of 
    TraceReport type of structures.
    :var serviceData: Service Data.
    """
    traceReports: List[TraceReport] = ()
    serviceData: ServiceData = None


@dataclass
class ValidateRevIdsInfo(TcBaseObj):
    """
    Input structure for 'validateRevIds' operation. It contains the revision id to be validated along with information
    about the object and object type.
    
    :var revId: Input Revision id to be validated.
    :var itemObject: Item object for which revision id needs to be validated. For new a Item to be created this
    property must be set to null.
    :var itemType: String describing the type of the Item for which the identifier is being validated.
    """
    revId: str = ''
    itemObject: Item = None
    itemType: str = ''


@dataclass
class ValidateRevIdsOutput(TcBaseObj):
    """
    This structure contains the modified revision id and enum status of the id.  Valid values for the enums are Valid
    (ids are valid), Invalid (ids are not valid), Modified (ids are not ideal but can be used if the user really wants
    them), Override (ids are not valid, silently replace with modified ones), and Duplicate (ids are already allocated
    in the system).
    
    :var modRevId: Modified rev id if specified by Naming Rules/Revision Naming Rules.
    :var revIdStatus: Status of the revision id represented as a 'ValidateRevIdsStatus' enum.
    """
    modRevId: str = ''
    revIdStatus: ValidateRevIdsStatus = None


@dataclass
class ValidateRevIdsResponse(TcBaseObj):
    """
    Response structure for 'validateRevIds' service operation.
    
    :var output: List of 'ValidateRevIdsOutput' structures, which contain the modified revision id and the validation
    status of the input revision id.
    :var serviceData: Standard 'ServiceData' structure. It contains partial errors and failures along with the
    clientIds.
    """
    output: List[ValidateRevIdsOutput] = ()
    serviceData: ServiceData = None


@dataclass
class DeepCopyData(TcBaseObj):
    """
    DeepCopyData definition
    
    :var attachedObject: Other side object
    :var propertyName: Name of relation type or reference property for which DeepCopy rule is configured
    :var propertyType: enumeration value indicating if DeepCopyRule is configured for Relation or Reference property
    :var copyAction: DeepCopy action [NoCopy, CopyAsReference, CopyAsObject or Select]
    :var isTargetPrimary: Flag indicating if target object is primary or secondary
    :var isRequired: If this flag is false, the copy action can be dynamicaly changed by user
    :var copyRelations: This is a Boolean representing whether the Properties on the Relation if any in the Relation
    that exists between
    :var saveAsInputTypeName: SaveAsInput type name
    :var childDeepCopyData: Vector of DeepCopy data for the secondary objects on the other side
    :var saveAsInput: SaveAsInput field to capture user inputs on the SaveAs dialog. NOTE: This field is unused in the
    getSaveAsDesc operation. It is used in the saveAsObjects operation
    """
    attachedObject: BusinessObject = None
    propertyName: str = ''
    propertyType: PropertyType = None
    copyAction: str = ''
    isTargetPrimary: bool = False
    isRequired: bool = False
    copyRelations: bool = False
    saveAsInputTypeName: str = ''
    childDeepCopyData: List[DeepCopyData] = ()
    saveAsInput: SaveAsInput = None


class PropertyType(Enum):
    """
    Map of the enumeration of the different property types for which DeepCopy Rule configuration is enabled.
    """
    Relation = 'Relation'
    Reference = 'Reference'


class ValidateRevIdsStatus(Enum):
    """
    Revision Id status enum after validation against Naming Rules/Revision Naming Rules.
    Meanings of the enum values are as follows:
    - ValidRevID            Revision Id is valid.
    - InvalidRevID            Revision Id is not valid. Do not allow the user to create a revision with this revision
    id.
    - ModifiedRevID        Revision Id is not valid. However, allow user to create revision with this id if they really
    want to.
    - OverrideRevID        Revision Id was not valid. Use the modified versions without telling the user. (Useful for
    case conversions)
    - DuplicateRevID        Revision Id is not valid, because it is already in use.
    
    """
    ValidRevID = 'ValidRevID'
    InvalidRevID = 'InvalidRevID'
    ModifiedRevID = 'ModifiedRevID'
    OverrideRevID = 'OverrideRevID'
    DuplicateRevID = 'DuplicateRevID'


"""
DateMap1
"""
DateMap1 = Dict[str, datetime]


"""
Map of DateTime array property names to values ('string, vector< DateTime>').
"""
DateVectorMap1 = Dict[str, List[datetime]]


"""
DoubleMap1
"""
DoubleMap1 = Dict[str, float]


"""
Map of double array property names to values ('string, vector< double>').
"""
DoubleVectorMap1 = Dict[str, List[float]]


"""
Map of float property names to values ('string, float').
"""
FloatMap1 = Dict[str, float]


"""
Map of float array property names to values ('string, vector<float>').
"""
FloatVectorMap1 = Dict[str, List[float]]


"""
Map of int property names to values ('string, int').
"""
IntMap1 = Dict[str, int]


"""
Map of int array property names to values ('string, vector< int>').
"""
IntVectorMap1 = Dict[str, List[int]]


"""
Map of BusinessObject property names to values ('string, BusinessObject').
"""
StringMap1 = Dict[str, str]


"""
Map of bool property names to values '(string, bool').
"""
BoolMap1 = Dict[str, bool]


"""
StringVectorMap1
"""
StringVectorMap1 = Dict[str, List[str]]


"""
TagMap1
"""
TagMap1 = Dict[str, BusinessObject]


"""
Map of BusinessObject array property names to values '(string, vector< BusinessObject >').
"""
TagVectorMap1 = Dict[str, List[BusinessObject]]


"""
Map of bool array property names to values ('string, vector< bool >').
"""
BoolVectorMap1 = Dict[str, List[bool]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, POM_object, Fnd0StaticTable
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class NameValueStruct1(TcBaseObj):
    """
    This structure contains property name and value pairs for each property.
    
    :var name: Name of the property
    :var values: Values of the property
    """
    name: str = ''
    values: List[str] = ()


@dataclass
class PostEventObjectProperties(TcBaseObj):
    """
    The 'PostEventObjectProperties' structure represents required parameters to post event on primaryObject when event
    eventTypeName occurs.
    
    :var clientId: A unique identifier supplied by the caller. This ID is used to identify return PostEventOutput and
    partial errors assocaited with this input structure. This is optional, provide empty String for null or optional
    value i.e. new String[0].
    :var primaryObject: The Business Object on which the event has occurred. This is a required parameter.
    :var secondaryObject: Secondary object should be passed when an event involves two objects, primaryObject and
    secondaryObject and writing the secondaryObject details conveys complete Audit information. Example, attaching
    license to Item. This is optional, provide null value for optional value.
    :var propertyCount: The propertyCount is the count of properties that user wants to be written  to Audit log. If
    the propertyCount is 0 the propertyNames and propertyValues will be ignored and treated as NULL values. This is
    optional and default value is 0.
    :var propertyNames: The propertyNames is the list of property names to be written to audit log. The total number of
    properties to write depends on the propertyCount value. This is optional, provide empty String for null or optional
    value i.e. new String[0].
    :var propertyValues: The propertyValues is the list of property values to be written to audit log for each of the
    propertyNames. The total number of properties to write depends on the propertyCount and propertyNames value. Any of
    these values if not specified will treat propertyValues as NULL.  This is optional, provide empty String for null
    or optional value i.e. new String[0].
    :var errorCode: Specify error code when failure of an event is to be recorded in audit log. This is optional and
    default value is 0.
    :var errorMessage: Specify error message when failure of an event is to be recorded in audit log. This is optional,
    you should provide empty String object for null or optional value i.e. new String[0].
    """
    clientId: str = ''
    primaryObject: BusinessObject = None
    secondaryObject: BusinessObject = None
    propertyCount: int = 0
    propertyNames: List[str] = ()
    propertyValues: List[str] = ()
    errorCode: int = 0
    errorMessage: str = ''


@dataclass
class PostEventOutput(TcBaseObj):
    """
    The 'PostEventOutput' structure represents the output success or failure for each of the
    'PostEventObjectProperties' structure in ifailError  for the assocaited clientId.
    
    :var clientId: Client unique identifier which is passed back for tracking the operation status.
    :var ifailError: The error code, if any. Packaged in the custom output response.
    """
    clientId: str = ''
    ifailError: int = 0


@dataclass
class PostEventResponse(TcBaseObj):
    """
    The 'PostEventResponse' structure represents the output returning a vector of 'PostEventOutput'  with partial
    errors wrapped in serviceData, if any.
    
    :var output: A vector of 'PostEventOutput' structures packaged in custom response. Success is defined by the return
    of the ifailError for post event on each of the primaryObject.
    :var serviceData: Partial failures will be returned in the Service Data for each failed processing. Error
    encountered while processing post event on element in the set is reported as partial errors and processing
    continues for the remaining elements in the input set.
    """
    output: List[PostEventOutput] = ()
    serviceData: ServiceData = None


@dataclass
class PropInfo(TcBaseObj):
    """
    This structure holds information about Teamcenter object & its timestamp and list of property name/value pair
    information.
    
    :var object: business object
    :var vecNameVal: Vector of property information
    :var timestamp: Timestamp of the object when object was exported to clients.
    """
    object: BusinessObject = None
    vecNameVal: List[NameValueStruct1] = ()
    timestamp: datetime = None


@dataclass
class RowData(TcBaseObj):
    """
    Row Data
    
    :var clientId: clientId
    :var rowObject: Row Object
    :var rowType: RowType
    :var rowAttrValueMap: Row Attribute Value Pair Map
    """
    clientId: str = ''
    rowObject: POM_object = None
    rowType: str = ''
    rowAttrValueMap: RowAttrValueMap = None


@dataclass
class SetPropertyResponse(TcBaseObj):
    """
    response structure for setProperties operation. It returns the information about overwritten objects.
    
    :var data: This is the service data. It contains the updated objects and their properties.
    :var objPropMap: Additional information to be communicated to client such as objects and props those are
    overwritten. This map can be empty if no overwritten object found or with QUERY option is not an input to the
    service operation.
    """
    data: ServiceData = None
    objPropMap: ObjectPropMap = None


@dataclass
class StaticTableDataResponse(TcBaseObj):
    """
    StaticTableDataResponse
    
    :var clientId: A unique string supplied by caller.
    This ID is used to identify return data elements and partial errors associated with this input structure. 
    
    :var serviceData: The Service Data.
    """
    clientId: str = ''
    serviceData: ServiceData = None


@dataclass
class StaticTableInfo(TcBaseObj):
    """
    Static Table Info
    
    :var tableType: type of table created/updated e.g. TableProperties
    :var tableObject: Fnd0StaticTable object
    """
    tableType: str = ''
    tableObject: Fnd0StaticTable = None


@dataclass
class VerifyExtensionInfo(TcBaseObj):
    """
    The required information in which to validate an extension exists on an operation for a specific type.
    
    :var typeName: The name of the type in which to check the operations.
    :var operationName: The name of the operation in which to check for the containing extension.
    :var extensionName: The name of the extension to check.
    :var extensionType: The extension type to check: 0=All, 1=PreCondition, 2=PreAction, 3=PostAction, 4=BaseAction
    """
    typeName: str = ''
    operationName: str = ''
    extensionName: str = ''
    extensionType: int = 0


@dataclass
class VerifyExtensionResponse(TcBaseObj):
    """
    The result of the Verify Extension method.
    
    :var output: Returns True if extension exists otherwise False.
    :var serviceData: This data structure provides service data for associated information.
    """
    output: List[bool] = ()
    serviceData: ServiceData = None


@dataclass
class CreateOrUpdateStaticTableDataResponse(TcBaseObj):
    """
    Contains Creation or updation response for Static table.
    
    :var serviceData: SOA Service Data.
    :var staticTableObject: StaticTable Object
    """
    serviceData: ServiceData = None
    staticTableObject: Fnd0StaticTable = None


@dataclass
class EventObject(TcBaseObj):
    """
    The 'EventObject' structure represents required parameter to get event type names for the businessObject.
    
    :var clientId: A unique identifier supplied by the caller. This ID is client's way of identifying event list.  This
    is a required parameter. If nothing is to be passed to clientId; assign an empty String object. Assigning NULL to
    clientId is not allowed.
    :var businessObject: The Business Object for which the valid Auditable and Subscribable event type list is to be
    retrieved. This is a required parameter.
    """
    clientId: str = ''
    businessObject: BusinessObject = None


@dataclass
class EventTypesOutput(TcBaseObj):
    """
    The 'EventTypesOutput' structure represents the outputs, auditableEvents and subscribableEvents, which are vectors
    of auditable event type names and subscribable event type names.
    
    :var clientId: Client unique identifier which is passed back for tracking the operation status.
    :var auditableEvents: The list of Auditable event type names.
    :var subscribableEvents: The list of Subscribable event type names
    """
    clientId: str = ''
    auditableEvents: List[str] = ()
    subscribableEvents: List[str] = ()


@dataclass
class EventTypesResponse(TcBaseObj):
    """
    The 'EventTypesResponse' structure represents the output response returning  a vector  of 'EventTypesOutput' with
    partial errors wrapped in  serviceData, if any.
    
    :var output: A vector of 'EventTypesOutput' structures packaged in custom response. Success is defined by the
    return of the 'ifailError' for getEventTypes on each of the 'businessObject'.
    :var serviceData: Partial failures will be returned in the ServiceDate for each failed processing. Error
    encountered while processing post event on element in the set is reported as partial errors and processing
    continues for the remaining elements in the input set.
    """
    output: List[EventTypesOutput] = ()
    serviceData: ServiceData = None


"""
This map has information about object and its properties.
"""
ObjectPropMap = Dict[BusinessObject, List[str]]


"""
A map of row attribute value pair. Value is vector to support typed reference attributes.
"""
RowAttrValueMap = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, WorkspaceObject
from enum import Enum
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class FindClassificationObjectsResponse(TcBaseObj):
    """
    Holds the classfication objects returned by the 'findClassificationObjects()' method.
    
    :var icos: Map of classified WorkspaceObjects and their associated Classification objects.
    :var data: Any failure will be returned with WorkspaceObject ID mapped to the error message in the ServiceData list
    of partial errors.
    """
    icos: WsoIcoMap = None
    data: ServiceData = None


@dataclass
class FindClassifiedObjectsResponse(TcBaseObj):
    """
    Holds the classified objects returned by the  'getFindClassifiedObjects()' method.
    
    :var wsos: List of classified WorkspaceObjects found for the given list of Classification objects.
    :var data: Any failure will be returned with WorkspaceObject ID mapped to the error message in the ServiceData list
    of partial errors.
    """
    wsos: List[WorkspaceObject] = ()
    data: ServiceData = None


@dataclass
class GetAttributesForClassesResponse(TcBaseObj):
    """
    Contains a list of Child Classes found for the given Classes
    
    :var attributes: Map of Class IDs and the Class attributes found for each of those classes
    :var data: Any failure will be returned with Class ID mapped to the error message in the ServiceData list of
    partial errors.
    """
    attributes: StrVClsAttrMap = None
    data: ServiceData = None


@dataclass
class GetChildrenResponse(TcBaseObj):
    """
    Contains a list of Child Classes found for the given Classes
    
    :var children: Map of Child classes found for input Class IDs.
    :var data: Any failures with Class ID mapped to the error message are returned in the ServiceData list of partial
    errors.
    """
    children: StrVChildDefMap = None
    data: ServiceData = None


@dataclass
class GetClassDescriptionsResponse(TcBaseObj):
    """
    Contains a list of Child Classes found for the given Classes.
    
    :var descriptions: Map of Child classes found for input Class IDs.
    :var data: Internal errors encounterd during the operation are added to the partial errors.
    """
    descriptions: StrClsDescMap = None
    data: ServiceData = None


@dataclass
class GetClassificationObjectsResponse(TcBaseObj):
    """
    Holds the classification object details returned by the 'GetClassificationObjects()' operation.
    
    :var clsObjs:  TagClsObjMap map of input classification object to its details.
    :var data: Any failure will be returned with Classification Object ID mapped to the error message in the
    'ServiceData' list of partial errors.
    """
    clsObjs: TagClsObjMap = None
    data: ServiceData = None


@dataclass
class GetFileIdAttributes(TcBaseObj):
    """
    Contains a list of child classes found for the given classes.
    
    :var relation: Name of the relation for attachement.
    :var datasetType: Internal type of the attached Dataset object.
    :var namedRefs: List of named references.
    """
    relation: str = ''
    datasetType: str = ''
    namedRefs: List[str] = ()


@dataclass
class GetFileIdCriteria(TcBaseObj):
    """
    Contains a list of child classes found for a class in the classification hierarchy.
    
    :var wsos: List of associated 'WorkspaceObjects' to execute the search for File IDs.
    :var criteria: Reference to the search criterion.
    """
    wsos: List[WorkspaceObject] = ()
    criteria: GetFileIdAttributes = None


@dataclass
class GetFileIdsResponse(TcBaseObj):
    """
    Contains a list of child classes found for the given classes
    
    :var ids: References to map of WorkspaceObject objects and attached Document objects.
    :var data: Internal errors encounterd during the operation are added to the partial errors.
    """
    ids: ObjTicketMap = None
    data: ServiceData = None


@dataclass
class GetKeyLOVsResponse(TcBaseObj):
    """
    Holds the key-LOV returned by the 'getKeyLOVs()' method.
    
    :var keyLOVs: Map of Key-LOV definitions details.
    :var data: Any failure will be returned with the Key-LOV ID mapped to the error message in the 'ServiceData' list
    of partial errors.
    """
    keyLOVs: StrKeyLOVDefMap = None
    data: ServiceData = None


@dataclass
class GetParentsResponse(TcBaseObj):
    """
    Holds the parents returned by the 'getParents()'' 'method.
    
    :var parents: Map contains a list of parents for each of the input Class IDs
    :var data: Internal errors encounterd during the operation are added to the partial errors.
    """
    parents: StrParentVecMap = None
    data: ServiceData = None


@dataclass
class GetPartFamilyTemplatesResponse(TcBaseObj):
    """
    Holds the PartFamilyTemplates returned by the 'getPartFamilyTemplates()'' 'method.
    
    :var wsos: Map of part family templates found for input Class IDs.
    :var data: Failures mapped to the error message in the 'ServiceData' list of partial errors.
    """
    wsos: StringPOMRefVecMap = None
    data: ServiceData = None


@dataclass
class KeyLOVDefinition(TcBaseObj):
    """
    Structure representing KeyLOVDefinition
    
    :var name: Name of the Key-LOV.
    :var options: Key-LOV options to Show/Hide keys.
    Valid values are  
    - 0 = Show key
    - 1 = Hide key
    
    
    :var keyValuePairs: Map of the Key-Value pairs.
    """
    name: str = ''
    options: int = 0
    keyValuePairs: StrStrMap = None


@dataclass
class AttributeFormat(TcBaseObj):
    """
    Structure representing format details of an attribute
    
    :var formatType: Integer representing the attribute type, which could be one of the following
    - - 1 => KeyLOV
    - 0 => String
    - 1 => Integer
    - 2 => Real
    - 3 => Date
    - 4 => Time range
    
    
    :var formatModifier1: Integer to indicate whether the attribute is configured for one of the following - 
    - 0 - Force positive number
    - 1 - Accept and display either + or -
    - 2 - Accept + or - but display - sign only
    
    
    Note: This field will return 0 if not applicable. Only applicable if the selected formatType is 1 or 2
    :var formatModifier2: Integer to indicate additional information about the selected formatType from one of the
    following : 
    If formatType = 0 then:
    - 0 = Print characters in upper and lower case
    - 1 = Print all characters in upper case
    - 2 = Print all characters in lower case
    
    
    If formatType = 2 then Number of digits after the decimal point
    
    Note: This field will return 0 if not applicable. Only applicable if the selected formatType is 0 or 2
    :var formatLength: Integer representing the length of the attribute. In case of a Key-LOV, this will contain the
    Key-LOV ID.
    """
    formatType: int = 0
    formatModifier1: int = 0
    formatModifier2: int = 0
    formatLength: int = 0


@dataclass
class SearchAttribute(TcBaseObj):
    """
    Structure SearchAttribute holds the attribute id and expression for attribute search
    
    :var attributeId: Alphanumeric attribute ID of the attribute used in this search.
    :var query: Query expression for this attribute e.g. >= 20.00. Supported Operators are: 
    - Equals - "="
    - Less than - "<"
    - Greater than - ">" 
    - Greater than or equal to - ">="
    - Less than or equal to - "<=",
    - OR - "|"
    - AND - "&"
    
    """
    attributeId: int = 0
    query: str = ''


@dataclass
class SearchByInstanceIdResponse(TcBaseObj):
    """
    Holds the Instance identifier returned by the 'getSearchByInstanceId'() method.
    
    :var clsObjTags: Retrieved classification objects.
    :var data: Any failure will be returned with classification object IDs mapped to the error message in the
    'ServiceData' list of partial errors.
    """
    clsObjTags: StrTagVecMap = None
    data: ServiceData = None


@dataclass
class SearchClassAttributes(TcBaseObj):
    """
    SearchClassAttributes holds classIds and expressions to search class attributes
    
    :var classIds: List of class IDs to perform search
    :var searchAttributes: List of references to the search attributes in these classes.
    :var searchOption: Unused parameter for this operation.
    """
    classIds: List[str] = ()
    searchAttributes: List[SearchAttribute] = ()
    searchOption: int = 0


@dataclass
class SearchForClassesCriteria(TcBaseObj):
    """
    Searches for classes based on the specified class attribute. Also allows sorting the results based on a predefined
    criterion. Search criteria must be specified and cannot be empty.
    
    :var searchAttribute: Class attribute to be searched for. Valid values are as defined in enum
    'ClassSearchAttribute'.
    :var searchString: Query string to search the class attribute by.
    :var sortOption: Option to sort the returned results. Valid values are as defined in enum 'ClassSortOption'.
    """
    searchAttribute: ClassSearchAttribute = None
    searchString: str = ''
    sortOption: ClassSortOption = None


@dataclass
class SearchForClassesResponse(TcBaseObj):
    """
    Returns a 'SearchForClassesResponse' structure containing:
    - Retrieved classes in the 'ServiceData' list of plain objects
    - Any failures with class ID mapped to the error message in the 'ServiceData' list of partial errors.
    
    
    
    :var classes: Map of search criteria index and the classes found for each of those searches.
    :var data: Internal errors encounterd during the operation are added to the partial errors
    """
    classes: IntClassDefVecMap = None
    data: ServiceData = None


@dataclass
class SearchResponse(TcBaseObj):
    """
    SearchResponse holds the response of search details
    
    :var clsObjTags: Map of the input query strings and references to the retrieved objects.
    :var data: Any failure will be returned mapped to the error message in the 'ServiceData' list of partial errors.
    """
    clsObjTags: StrTagVecMap = None
    data: ServiceData = None


@dataclass
class TypedDocument(TcBaseObj):
    """
    Structure holding the attached Icon, Image or NamedRef or this child class.
    
    :var documentType: Contains type of document attached. Valid values are:
    - icon
    - image
    - NamedRef
    
    
    :var ticket: Ticket identifier for the attached file
    :var originalFileName: File name for this attachment.
    """
    documentType: str = ''
    ticket: str = ''
    originalFileName: str = ''


@dataclass
class UpdateClassificationObjectsResponse(TcBaseObj):
    """
    UpdateClassificationObjectsResponse
    
    :var clsObjs: List of updated Classification objects.
    :var data: Any failure will be returned with Classification object ID mapped to the error message in the
    'ServiceData' list of partial errors.
    """
    clsObjs: List[ClassificationObject] = ()
    data: ServiceData = None


@dataclass
class ChildDef(TcBaseObj):
    """
    Structure representing Child Definition
    
    :var id: Child class ID.
    :var type: Type of Class. Valid values are -
    - Group                 = (1 << 0)
    - Class                  = (1 << 1)
    - View                   = (1 << 2)
    - Storage class     = (1 << 4)
    
    
    :var name: Name of the child class
    :var description: Description of the child class
    :var childCount: Number of children underneath this child class.
    :var instanceCount: Number of instances of classification objects in this child class and its descendents.
    :var viewCount: Number of views for this child class.
    :var documents: List of references to the 'TypedDocument' structure containing Documents attached to this child
    class.
    """
    id: str = ''
    type: str = ''
    name: str = ''
    description: str = ''
    childCount: int = 0
    instanceCount: int = 0
    viewCount: int = 0
    documents: List[TypedDocument] = ()


@dataclass
class ClassAttribute(TcBaseObj):
    """
    Contains a list of Child Classes found for the given Classes
    
    :var id: Integer ID for this attribute
    :var name: Name for this attribute
    :var description: Description added for this attribute.
    :var arraySize: Array size or the number of values for this attribute
    - If single valued (non array), then 'arraySize' = 1
    - If multi valued (array), then 'arraySize' >= 1 corresponding to the size of the array defined in the attribute
    definition.
    
    
    :var options: Attribute property flags represented as a single integer.
    To access individual property flags, a bitwise operations will be required by the client.
    Valid values are:
    - ATTR_vla                      = (1 << 0)
    - ATTR_external_vla      = (1 << 1)
    - ATTR_mandatory         = (1 << 2)
    - ATTR_protected             = (1 << 3)
    - ATTR_unique                 = (1 << 4)
    - ATTR_propagated         = (1 << 5)
    - ATTR_localValue             = (1 << 6)
    - ATTR_reference             = (1 << 7)
    - ATTR_auto_computed = (1 << 15)
    - ATTR_hidden                = (1 << 20)
    - ATTR_localizable        = (1 
    
    
    :var preConfig: Preconfiguration attached to the attribute.
    :var config: Base configuration attached to the attribute
    :var postConfig: Post configuration attached to the attribute.
    Configurations could be any combinations of the following individual configurations:
    - multifield
    - horizontal
    - vertical
    - separator
    - arrow
    - button
    - wide
    - mandatory flag
    - protected flag
    - unique flag
    
    
    :var shortName: Short name defined for this attribute
    :var annotation: Annotation information added to this attribute
    :var format: References the 'FormatProperties' structure defining the metric format of this attribute.
    :var altFormat: References the FormatProperties structure defining the nonmetric format of this attribute.
    :var unitName: Unit display name associated with this attribute in this unit system.Note: Only applicable to
    numerical formats of attributes.
    :var defaultValue: Default value of this Class attribute. This can be an empty string indicating no default value.
    :var minValue: Minimum value constraint of this Class attribute. This can be an empty string indicating no minimum
    value constraint.
    Note: Only applicable to numerical formats of attributes.
    :var maxValue: Maximum value contraint of this Class attribute. This can be an empty string indicating no maximum
    value constraint.
    Note: Only applicable to numerical formats of attributes.
    """
    id: int = 0
    name: str = ''
    description: str = ''
    arraySize: int = 0
    options: int = 0
    preConfig: str = ''
    config: str = ''
    postConfig: str = ''
    shortName: str = ''
    annotation: str = ''
    format: AttributeFormat = None
    altFormat: AttributeFormat = None
    unitName: str = ''
    defaultValue: str = ''
    minValue: str = ''
    maxValue: str = ''


@dataclass
class ClassDef(TcBaseObj):
    """
    Properties of the given class.
    
    :var id: Class ID
    :var parent: Class ID of the parent class.
    :var instanceCount: Total number of classification objects instantiated in this class or any of its descendents
    :var viewCount: Number of 'Views' defined for this class
    :var documents: List of attached 'Icons', 'Images' and 'NamedRefs' to this class
    :var name: Class name
    :var shortName: Short name
    :var description: Class description
    :var unitBase: Unit system of the class. See enum 'UnitBase' for list of valid values
    :var options: Class options as specified in the enum 'ClassFlags' below
    :var userData1: User data 1 added to this class
    :var userData2: User data 2 added to this class
    :var childCount: Number of child classes for this class
    """
    id: str = ''
    parent: str = ''
    instanceCount: int = 0
    viewCount: int = 0
    documents: List[TypedDocument] = ()
    name: str = ''
    shortName: str = ''
    description: str = ''
    unitBase: UnitBase = None
    options: ClassFlags = None
    userData1: str = ''
    userData2: str = ''
    childCount: int = 0


@dataclass
class ClassFlags(TcBaseObj):
    """
    Contains a list of child classes found for the given classes.
    
    :var isValid: - True value indicates that the class is a valid class. The option is for future use and currently is
    set by default to true internally.
    
    
    :var isAbstract: True value indicates that the class is abstract class, else storage class.
    :var isGroup: True value indicates that the class is a group.
    :var isAssembly: True value indicates that the class is an assembly.
    """
    isValid: bool = False
    isAbstract: bool = False
    isGroup: bool = False
    isAssembly: bool = False


@dataclass
class ClassificationObject(TcBaseObj):
    """
    Structure representing Classification Object details
    
    :var clsObjTag: Reference of Classification object.
    :var instanceId: Alphanumeric ID of the Classification object.
    :var classId: Unique Alphanumeric ID of the Classification class where this object was created.
    :var unitBase: Unit system of measure in which the Classification object is stored in.
    :var wsoId: Reference of the WorkspaceObject (WSO) that is associated by this Classification object. This can be
    empty. Allowed WSO types will be defined by the preference 'ICS_allowable_types'.
    
    :var properties: Array of Classification attributes references that store the properties of this Classification
    object.
    """
    clsObjTag: BusinessObject = None
    instanceId: str = ''
    classId: str = ''
    unitBase: UnitBase = None
    wsoId: WorkspaceObject = None
    properties: List[ClassificationProperty] = ()


@dataclass
class ClassificationProperty(TcBaseObj):
    """
    Structure representing Classification Property which holds attribute ids  and their values
    
    :var attributeId: Holds the unique identifier of an attribute.
    :var values: Holds a array of values for this attribute in the context of a Classification object.
    [Note: An array is required as an attribute can be single or multi-valued.]
    
    """
    attributeId: int = 0
    values: List[ClassificationPropertyValue] = ()


@dataclass
class ClassificationPropertyValue(TcBaseObj):
    """
    Holds the 'DB value' and  an optional 'Display value' of a property
    
    :var dbValue: Value of the Classification attribute as stored in the database.
    :var displayValue: Value of the Classification attribute as  it should displayed in the client. 
    [ For example, If the attribute is a Key-LOV then the Key is stored as the 'dbValue', while the 'displayValue' can
    be configured to be either of the following based on the Key-Value pairs in the Key-LOV definition: 
    - "Value" only.
    - Concatenated "Key" and "Value". ]
    
    """
    dbValue: str = ''
    displayValue: str = ''


@dataclass
class CreateClassificationObjectsResponse(TcBaseObj):
    """
    Holds the classification objects returned by the 'getCreateClassificationObjects()' method.
    
    :var clsObjs: List of created Classification objects.
    :var data: Any failure will be returned with Classification object ID mapped to the error message in the
    ServiceData list of partial errors.
    """
    clsObjs: List[ClassificationObject] = ()
    data: ServiceData = None


class ClassSearchAttribute(Enum):
    """
    Enumerates the class attributes available for searching.
    
    :var CSA_CLASSID: The ID of the Classification class.
    :var CSA_CLASSNAME: The name of the Classification class.
    :var CSA_CLASSTYPE: The type of the Classification class.
    :var CSA_SHORTNAME: The short name of the Classification class.
    :var CSA_GROUPID: The group ID of the Classification class.
    :var CSA_USERDATA1: User data 1 added to the class.
    :var CSA_USERDATA2: User data 2 added to the class.
    :var CSA_ATTRID: The attribute id of the Classification class.
    :var CSA_ATTRNAME: The attribute name of the Classification class.
    """
    CSA_CLASSID = 'CSA_CLASSID'
    CSA_CLASSNAME = 'CSA_CLASSNAME'
    CSA_CLASSTYPE = 'CSA_CLASSTYPE'
    CSA_SHORTNAME = 'CSA_SHORTNAME'
    CSA_GROUPID = 'CSA_GROUPID'
    CSA_USERDATA1 = 'CSA_USERDATA1'
    CSA_USERDATA2 = 'CSA_USERDATA2'
    CSA_ATTRID = 'CSA_ATTRID'
    CSA_ATTRNAME = 'CSA_ATTRNAME'


class ClassSortOption(Enum):
    """
    Enumerates the sort options available for listing the search results.
    
    :var CSO_CLASSID: The ID of the Classification class.
    :var CSO_CLASSNAME: The name of the Classification class.
    :var CSO_CLASSTYPE: The type of the Classification class.
    :var CSO_SHORTNAME: The short name of the Classification class.
    :var CSO_GROUPID: The group ID of the Classification class.
    :var CSO_USERDATA1: User data 1 added to the class.
    :var CSO_USERDATA2: User data 2 added to the class.
    :var CSO_ATTRID: The attribute ID of the Classification class.
    :var CSO_ATTRNAME: The attribute name of the Classification class.
    """
    CSO_CLASSID = 'CSO_CLASSID'
    CSO_CLASSNAME = 'CSO_CLASSNAME'
    CSO_CLASSTYPE = 'CSO_CLASSTYPE'
    CSO_SHORTNAME = 'CSO_SHORTNAME'
    CSO_GROUPID = 'CSO_GROUPID'
    CSO_USERDATA1 = 'CSO_USERDATA1'
    CSO_USERDATA2 = 'CSO_USERDATA2'
    CSO_ATTRID = 'CSO_ATTRID'
    CSO_ATTRNAME = 'CSO_ATTRNAME'


class UnitBase(Enum):
    """
    UNSPECIFIED Both or no unit system of measure.
    METRIC Metric unit system of measure.
    ENGLISH Non-metric unit system of measure.
    """
    UNSPECIFIED = 'UNSPECIFIED'
    METRIC = 'METRIC'
    ENGLISH = 'ENGLISH'


"""
Map of search criteria index and the classes found for each of those searches
"""
IntClassDefVecMap = Dict[int, List[ClassDef]]


"""
References to map of WorkspaceObject objects and attached Document objects.
"""
ObjTicketMap = Dict[WorkspaceObject, List[TypedDocument]]


"""
Map of Class IDs and their respective child Class IDs
"""
StrClsDescMap = Dict[str, ClassDef]


"""
Structure elements:

- keyLOVs - Map of Key-LOV definitions details.
- data     - Any failure will be returned with the Key-LOV ID mapped to the error message in the 'ServiceData' list of partial errors.


"""
StrKeyLOVDefMap = Dict[str, KeyLOVDefinition]


"""
Map contains a list of parents Class IDs for each of the input Class ID.
"""
StrParentVecMap = Dict[str, List[str]]


"""
Structure elements:

- primary key - String representing a key of a Key-LOV entry. 
- value - String representing a value  of the Key-LOV  entry.


"""
StrStrMap = Dict[str, str]


"""
Maps the query string and the resulting Objects (identified by a tag)

Structure elements:

- primary key - Input query string used for this search.
- values - References of the objects found for this query string.


"""
StrTagVecMap = Dict[str, List[BusinessObject]]


"""
Map of Class IDs and their respective child Class IDs. For every Class ID, there will be a list of Child Class IDs.
"""
StrVChildDefMap = Dict[str, List[ChildDef]]


"""
Contains a list of Child Classes found for the given Classes.

Structure elements:

- Primary key : Alpha-numeric Class ID for which class attributes are being asked for.
- Values : List of references to ClassAttribute structures.


"""
StrVClsAttrMap = Dict[str, List[ClassAttribute]]


"""
Structure elements:

- primary key - Class IDs.
- values - List of the retreived part family template references


"""
StringPOMRefVecMap = Dict[str, List[WorkspaceObject]]


"""
Map of Classification Object tag to the details of Classification Object.


- Key - Classification Object tag.
- Value - ClassificationObject structure representing Classification Object details.


"""
TagClsObjMap = Dict[BusinessObject, ClassificationObject]


"""
- primary key : Reference of the classified WorkspaceObject  that was used to find the classification objects.
- Values : List of Classification object references associated with the classified WorksoaceObject referenced in the primary key


"""
WsoIcoMap = Dict[WorkspaceObject, List[BusinessObject]]



from __future__ import annotations

from tcsoa.gen.AWS2._2017_06.UiConfig import ColumnConfigData
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetOrResetUIColumnConfigResponse(TcBaseObj):
    """
    This structure returns information to the client about column configurations. The ServiceData contains information
    about errors encountered during processing.
    
    :var clientName: The client for which the configurations are applicable.
    :var columnConfigurations: List of column configurations data.
    :var serviceData: ServiceData structure containing errors and command, command collection and icon objects. If
    there is an error retrieving the configuration information, the error added to the ServiceData as a partial error.
    """
    clientName: str = ''
    columnConfigurations: List[ColumnConfigData] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from tcsoa.gen.BusinessObjects import VerificationRule
from dataclasses import dataclass


@dataclass
class GetVerificationRulesResponse(TcBaseObj):
    """
    The response information returned by getVerificationRules.
    
    :var serviceData: The ServiceData object  which contains generic information like error number and error
    information.
    :var outputs: The list of VerificationRule objects information.
    """
    serviceData: ServiceData = None
    outputs: List[VerificationRulesInfo] = ()


@dataclass
class VerificationRuleInput(TcBaseObj):
    """
    The criteria to get VerificationRule objects.
    
    :var functionality: The desired value of the functionality property of the VerificationRule objects.
    :var conditionName: The desired value of the condition_reference property of the VerificationRule objects.
    :var typeName: The desired value (business object type name) of the type property of the VerificationRule objects.
    :var subGroup: The desired value of the subGroup property of the VerificationRule objects. It should be the value
    of LOV in subGroupLOV  property of FunctionalityRule object associated with the VerificationRule object.
    """
    functionality: str = ''
    conditionName: str = ''
    typeName: str = ''
    subGroup: str = ''


@dataclass
class VerificationRulesInfo(TcBaseObj):
    """
    The returned VerificationRule objects and it associated VerificationRuleInput index.
    
    :var index: The index of VerificationRuleInput in parameter vector that this list of rules is associated with.
    :var verificationRules: The list of VerificationRule objects which match the desired criteria.
    """
    index: int = 0
    verificationRules: List[VerificationRule] = ()



from tcsoa.gen.Internal_Cad._2007_12.services import DataManagementService as imp0
from tcsoa.gen.Internal_Cad._2008_03.services import DataManagementService as imp1
from tcsoa.gen.Internal_Cad._2017_05.services import StructureManagementService as imp2
from tcsoa.gen.Internal_Cad._2013_05.services import DataManagementService as imp3
from tcsoa.gen.Internal_Cad._2008_06.services import DataManagementService as imp4
from tcsoa.gen.Internal_Cad._2008_05.services import DataManagementService as imp5
from tcsoa.gen.Internal_Cad._2010_04.services import DataManagementService as imp6
from tcsoa.utils import TcService


class DataManagementService(TcService):
    createAppUidObjects = imp0.createAppUidObjects
    exportConfiguredNXAssembly = imp1.exportConfiguredNXAssembly
    queryPartRelatedFeatures = imp3.queryPartRelatedFeatures
    queryRelatedFeatures = imp4.queryRelatedFeatures
    resolveAttrMappingsForNX = imp5.resolveAttrMappingsForNX
    revise = imp6.revise
    saveAsNewItem = imp6.saveAsNewItem


class StructureManagementService(TcService):
    getProductStructureArrangements = imp2.getProductStructureArrangements



from __future__ import annotations

from tcsoa.gen.Internal.BusinessModeler._2013_05.DynamicLOVQuery import QueryData, LOVSearchResults
from tcsoa.utils import TcService


class DynamicLOVQueryService(TcService):

    @classmethod
    def executeDynamicLOVQuery(cls, queryData: QueryData) -> LOVSearchResults:
        """
        This operation queries for a subset of LOV values based on the desired query constraints. The dynamic LOV
        definition including the query string, query class and the properties representing the value and description
        are passed to this operation. Apart from this filter information (LOVFilterData) are passed to this service
        which is used to define the output (no of output, order of output information) of this service operation. This
        service operation executes the LOV query to database every time when the QueryData:: unprocessedUIDs list is
        empty otherwise it process the next set of UIDs from the list of unprocessed UIDs and send it back to the
        client.
        
        Exceptions:
        >ServiceException :
        - 226576 - The query for the dynamic LOV has failed to execute. Please refer to the Teamcenter server syslog
        file for details, and check the query attributes.
        - 226577 - An error has occurred while retrieving properties values. Please contact your Teamcenter
        administrator.
        
        """
        return cls.execute_soa_method(
            method_name='executeDynamicLOVQuery',
            library='Internal-BusinessModeler',
            service_date='2013_05',
            service_name='DynamicLOVQuery',
            params={'queryData': queryData},
            response_cls=LOVSearchResults,
        )



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GlobalConstantValue2(TcBaseObj):
    """
    Holds the name of the global constant and corresponding global constant value.
    
    :var key: Name of the global constant.
    :var value: The global constant value(s) corresponding to the specified constant.
    """
    key: str = ''
    value: List[str] = ()


@dataclass
class GlobalConstantValueResponse2(TcBaseObj):
    """
    Holds the response for the 'getGlobalConstantValues' operation.
    
    :var constantValues: The requested global constants.
    :var serviceData: This contains the status of the operation. A partial error is returned If the name global
    constant cannot be added to the global default cache (74502).
    """
    constantValues: List[GlobalConstantValue2] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.ProjectManagement._2012_09.ScheduleManagement import AssignmentCreateContainer
from tcsoa.gen.ProjectManagement._2012_02.ScheduleManagement import CreatedObjectsContainer
from typing import List
from tcsoa.gen.BusinessObjects import ScheduleTask, Schedule, ResourceAssignment
from tcsoa.utils import TcService


class ScheduleManagementService(TcService):

    @classmethod
    def claimAssignment(cls, task: ScheduleTask, assignment: ResourceAssignment) -> CreatedObjectsContainer:
        """
        Claims an assignment
        
        Exceptions:
        >None
        """
        return cls.execute_soa_method(
            method_name='claimAssignment',
            library='ProjectManagement',
            service_date='2012_09',
            service_name='ScheduleManagement',
            params={'task': task, 'assignment': assignment},
            response_cls=CreatedObjectsContainer,
        )

    @classmethod
    def assignResources(cls, schedule: Schedule, createAssignments: List[AssignmentCreateContainer]) -> CreatedObjectsContainer:
        """
        Assigns resources to tasks.
        
        Exceptions:
        >None
        """
        return cls.execute_soa_method(
            method_name='assignResources',
            library='ProjectManagement',
            service_date='2012_09',
            service_name='ScheduleManagement',
            params={'schedule': schedule, 'createAssignments': createAssignments},
            response_cls=CreatedObjectsContainer,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class WorkflowTaskViewModelInput(TcBaseObj):
    """
    WorkflowTaskViewModelInput stucture that represent the input EPMTask or Signoff and its properties.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    :var selection: The selected EPMTask or Signoff object data for which the information to be sent in a JSON format.
    :var typeToPropertiesMap: A map (string, list of strings) represents the different properties of the task with type
    name. This input specify the properties of the  task to be outputted to JSON string.
    If a client wants to get a property value of an object which can be traversed using a reference property on a given
    input task then following format need to be used :
    REF( reference_property_name, reference_object_type ).property_on_reference_object
    
    For example : If the client wants to get object_string of workflow process (EPMJob), then it has to provide the
    input data in following format : 
    "EPMTask", "REF(parent_process,EPMJob).object_string"}
    
    In above example :
    reference_property_name is the "parent_process" reference    property defined on EPMTask.
    reference_object_type is the "EPMJob" referenced by the "parent_process" property on EPMTask.
    property_on_reference_object  is the "object_string". property on EPMJob.
    
    Below are the few examples of this input:
    To get task properties: "EPMTask", {"state_value", "signoff_quorum"}
    To get task reference property :"EPMTask", {"REF(parent_process,EPMJob).object_string"}
    To get task signoff data : "Signoff", {"REF(group_member,GroupMember).group", "REF(group_member,GroupMember).user",
    "fnd0DecisionRequired" }
    To get task signoff profile data : "EPMSignoffProfile",{"REF(group,Group).name", "REF(role,Role).role_name",
    "number_of_signoffs"}
    """
    clientId: str = ''
    selection: BusinessObject = None
    typeToPropertiesMap: KeyValueMap = None


@dataclass
class WorkflowTaskViewModelOutput(TcBaseObj):
    """
    Output structure represents the task data in JSON string and the additional data if any.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    :var additionalData: A map (string, list of strings) represents the additional information required by client.For
    example this operation returns logical flag to specify if the complete button can be enabled for the selected task
    in the perform dialog.
    :var taskJSONStrings: A map (string, list of strings) represents the additional information required by client.For
    example this operation returns logical flag to specify if the complete button can be enabled for the selected task
    in the perform dialog.
    """
    clientId: str = ''
    additionalData: KeyValueMap = None
    taskJSONStrings: StringMap = None


@dataclass
class WorkflowTaskViewModelResponse(TcBaseObj):
    """
    response
    
    :var output: response.
    :var serviceData: Service data.
    """
    output: List[WorkflowTaskViewModelOutput] = ()
    serviceData: ServiceData = None


"""
KeyValueMap.
"""
KeyValueMap = Dict[str, List[str]]


"""
String Map.
"""
StringMap = Dict[str, str]
